SystemOrganization addCategory: #Flow!
SystemOrganization addCategory: #'Flow-Errors'!
SystemOrganization addCategory: #'Flow-Announcements'!
SystemOrganization addCategory: #'Flow-Handlers'!
SystemOrganization addCategory: #'Flow-Models'!
SystemOrganization addCategory: #'Flow-Libraries'!
SystemOrganization addCategory: #'Flow-Core'!

Exception subclass: #FlowError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Errors'!

FlowError subclass: #FlowModelDataError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Errors'!

FlowError subclass: #FlowModelGetterIsMissing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Errors'!

FlowError subclass: #FlowModelMetadataError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Errors'!

WARestfulHandler subclass: #FlowAPIHandler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowAPIHandler commentStamp: 'sebastiansastre 3/9/2012 07:22' prior: 0!
AmberMPRestfulHandler lays the foundation to maintain ProxyModel and PersistentModel objects between client and server!

!FlowAPIHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 3/9/2012 10:28'!
basePath
	"This is the base path (first URI dir name) of the URL used by this API"
	
	^ 'api'
! !

!FlowAPIHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 3/11/2012 22:49'!
database 

	^ FlowSimpleDb instance ! !

!FlowAPIHandler class methodsFor: 'initialization' stamp: 'sebastiansastre 3/10/2012 21:19'!
initialize

	self allSubclasses do:[:class|
		class initialize]! !

!FlowAPIHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 3/9/2012 18:09'!
modelClass 
	"Answers the class of the models manipulated by this handler.
	Suggested practice is: one handler per class model."
	
	self subclassResponsibility ! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 12:04'!
answerJSONFor: anObject
	"Produce the http response with anObject as JSON."
	
	self requestContext
		respond: [ :response | 
			response
				document: anObject asJsonString
				mimeType: 'text/plain']! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 09:12'!
commit: aPath package: aFilename

	| filename code |

	filename := (FSRelativePath withAll: (aPath subStrings: '/')) / aFilename.	
	code := self requestContext request rawBody.

	(self libraryFor: filename) write: code on: filename.

	self requestContext
		respond: [ :response | 
			response status: response class statusOk ]! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 20:40'!
create
	<post>
	<path: '/'>
	
	| newOne |
	
	newOne := self newModel.

	self onBeforeCreate: newOne.

	^ self answerJSONFor: newOne
	! !

!FlowAPIHandler methodsFor: 'accessing' stamp: 'sebastiansastre 3/10/2012 11:47'!
database 

	^ self class database ! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/24/2012 12:11'!
delete: anId
	<delete>
	<path: '/{anId}'>
		
	| aModel |
	aModel := self readModel: anId.

	self onBeforeDelete: aModel.
	
	^ self answerJSONFor: aModel! !

!FlowAPIHandler methodsFor: 'accessing' stamp: 'sebastiansastre 3/9/2012 18:08'!
modelClass

	^ self class modelClass! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/10/2012 11:50'!
newModel

	^ self modelClass new ! !

!FlowAPIHandler methodsFor: 'reactions' stamp: 'sebastiansastre 3/15/2012 21:35'!
onBeforeCreate: newOne
	"This model has been created (and is about to be stored).
	This is a great chance to do something you could need about this."
	
	self database create: newOne
! !

!FlowAPIHandler methodsFor: 'reactions' stamp: 'sebastiansastre 3/15/2012 21:33'!
onBeforeDelete: aModel
	"This model is about to be deleted.
	This is a great chance to do something you could need about this."
	
	self database delete: aModel! !

!FlowAPIHandler methodsFor: 'reactions' stamp: 'sebastiansastre 3/27/2012 10:41'!
onBeforeRead: anId
	"This model is about to be read.
	This is a great chance to do something you could need about this."

	^ (self modelClass atId: anId) onRead! !

!FlowAPIHandler methodsFor: 'reactions' stamp: 'sebastiansastre 3/15/2012 21:33'!
onBeforeUpdate: aModel
	"This model is about to be updated.
	This is a great chance to do something you could need about this."
	
	self database update: aModel! !

!FlowAPIHandler methodsFor: 'reactions' stamp: 'sebastiansastre 3/15/2012 23:09'!
onBeforeUpdateFrom: someJson
	"A model is about to be updated.
	This is a great chance to do something you could need about this."

	| aModel |

	aModel := self reify: someJson.
	
	^ self database update: aModel! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 12:03'!
read: anId
	<get>
	<path: '/{anId}'>
	
	^ self answerJSONFor: (self readModel: anId)! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:38'!
readModel: anId
	"Answers the persisted instance of the model found at anId (or nil)"

	^ self onBeforeRead: anId! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/17/2012 15:13'!
reify: someJson
	"Answers a model reified from someJson.
	AKA inflates that JSON :)"

	^ self modelClass fromJson: someJson! !

!FlowAPIHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/27/2012 10:19'!
update: anId
	<put>
	<path: '/{anId}'>
	
	| aModel json |
	json := 	self requestContext request rawBody.

	aModel := self onBeforeUpdateFrom: json.
	 	
	^ self answerJSONFor: aModel! !

FlowAPIHandler subclass: #FlowCodeHandler
	instanceVariableNames: ''
	classVariableNames: 'ProjectLibrary'
	poolDictionaries: ''
	category: 'Flow-Handlers'!

!FlowCodeHandler commentStamp: 'sebastiansastre 3/15/2012 16:31' prior: 0!
FlowCodeHandler 
		
This class handles your Amber's code.

Your code is treated differently depending if ti's about Amber itself or if it's about your project.

For Amber itself, this handler uses FlowLibrary and for your project it keeps a class var ProjectLibrary to manage it with your own library.
Note that it's always expected that you put your own library there.

Your own libraries for your own projects are the right place to configure the path where your Amber's application files are gowing to be read and write. Note that Flow manages the code for only one project at the time. If you have many projects in the same image you need to change it accordingly.

Example:

	FlowCodeHandler projectLibrary: MyGreatAmberProjectLibrary default


MyGreatAmberProjectLibrary class>>basePath
	
	^ '/Users/sebastian/Documents/git/myGreatAmberProject' asPath  
	
...and

FlowLibrary class>>basePath
	
	^ '/Users/sebastian/Documents/git/amber' asPath    <-- that's where you have your local git clone of your forked amber
!

!FlowCodeHandler class methodsFor: 'initialization' stamp: 'sebastiansastre 3/15/2012 17:48'!
initialize

	WAAdmin register: self at: 'flow'! !

!FlowCodeHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 3/15/2012 16:20'!
projectLibrary
	
	^ ProjectLibrary ifNil:[self error: 'Please define the library for your project (see class comments for details)']! !

!FlowCodeHandler class methodsFor: 'accessing' stamp: 'sebastiansastre 3/15/2012 16:20'!
projectLibrary: aFileSystemLibrary
	
	^ ProjectLibrary := aFileSystemLibrary! !

!FlowCodeHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 17:50'!
commitJSPackage: packageName
	<put>
	<path: '/commit/{packageName}.js'>

	self commit: 'js' package: packageName,'.js'
! !

!FlowCodeHandler methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 17:50'!
commitSTPackage: packageName
	<put>
	<path: '/commit/{packageName}.st'>

	self commit: 'st' package: packageName,'.st'
! !

!FlowCodeHandler methodsFor: 'testing' stamp: 'sebastiansastre 3/18/2012 09:04'!
isAmberFile: aFilename
	"Answers true if aFilename exists in the Amber repository (false otherwise)"
	
	| absolute |
	
	absolute := FlowAmberLibrary basePath. 
	aFilename segments do:[:part|
		absolute := absolute / part].
	
	^ FlowAmberLibrary files includes: absolute asReference ! !

!FlowCodeHandler methodsFor: 'testing' stamp: 'sebastiansastre 3/18/2012 09:14'!
isFlowFile: aFilename
	"Answers true if aFilename exists in the Amber repository (false otherwise)"
	
	| absolute |
	
	absolute := FlowLibrary basePath. 
	aFilename segments do:[:part|
		absolute := absolute / part].
	
	^ (FlowLibrary files includes: absolute asReference) or:[
	('Flow-*' match: aFilename basename)]! !

!FlowCodeHandler methodsFor: 'accessing' stamp: 'sebastiansastre 3/15/2012 16:14'!
library
	"Answers the library that should handle requests to static content."
	
! !

!FlowCodeHandler methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 10:55'!
libraryFor: aFilename
	"Answers the library that should handle requests to static content.
	This handler assumes that if the file is NOT among Amber's files, then it belongs to your project."

	^ (self isAmberFile: aFilename)
		ifTrue:[FlowAmberLibrary default]
		ifFalse:[
			(self isFlowFile: aFilename) 
				ifTrue:[FlowLibrary default]
				ifFalse:[ProjectLibrary]]
	
! !

FileSystemLibrary subclass: #FlowAbstractLibrary
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Libraries'!

FlowAbstractLibrary class
	instanceVariableNames: 'files'!

FlowAbstractLibrary class
	instanceVariableNames: 'files'!

!FlowAbstractLibrary class methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 08:55'!
basePath
	"Answers the path where you store your local git clone of amber or flow.
	Note: if you don't use git (you should), it's probably okay to use a downloaded version."
	
	
	self subclassResponsibility ! !

!FlowAbstractLibrary class methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 08:55'!
files 

	^ files ifNil:[self initializeFiles]! !

!FlowAbstractLibrary class methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 08:52'!
getFiles 

	^ self basePath asReference allChildren 
	! !

!FlowAbstractLibrary class methodsFor: 'initialization' stamp: 'sebastiansastre 3/18/2012 08:52'!
initialize 

	Smalltalk 
		addToStartUpList: self;
		addToShutDownList: self	! !

!FlowAbstractLibrary class methodsFor: 'initialization' stamp: 'sebastiansastre 3/18/2012 08:55'!
initializeFiles 

	^ files := self getFiles! !

!FlowAbstractLibrary class methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 08:55'!
reset 

	files := nil! !

!FlowAbstractLibrary class methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 08:52'!
shutDown 

	self reset! !

!FlowAbstractLibrary class methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 08:52'!
startUp: resuming

	self reset! !

FlowAbstractLibrary subclass: #FlowAmberLibrary
	instanceVariableNames: ''
	classVariableNames: 'Files'
	poolDictionaries: ''
	category: 'Flow-Libraries'!

!FlowAmberLibrary class methodsFor: 'accessing' stamp: 'paulwilke 5/2/2012 15:31'!
basePath
	"Answers the path where you store your local git clone of amber.
	Note: if you don't use git (you shuld), that could be your downloaded version."
	
	"Remove this transcript show: after customizing your path or..
	better, override this as an external method for some of your own packages."
	
	"Transcript cr; show: 'Check FlowAmberLibrary class>>basePath it should be pointing to a fresh Amber''s release'."
		
		
	Author fullName = 'sebastiansastre' ifTrue:[
		^ '/Users/sebastian/Documents/git/amber' asPath].
	
	Author fullName = 'paulwilke' ifTrue:[
		^ 'C:\Users\paul\Development\GitRepo\amber' asPath]  	! !

FlowAbstractLibrary subclass: #FlowLibrary
	instanceVariableNames: ''
	classVariableNames: 'Files'
	poolDictionaries: ''
	category: 'Flow-Libraries'!

!FlowLibrary class methodsFor: 'accessing' stamp: 'sebastiansastre 5/31/2012 23:45'!
basePath
	"Answers the path where you store your local git clone of amber.
	Note: if you don't use git (you shuld), that could be your downloaded version."
	
	"Remove this transcript show: after customizing your path or..
	better, override this as an external method for some of your own packages."
	
	"Transcript cr; show: 'Check FlowLibrary class>>basePath it should be pointing to a fresh flow''s release'."
		
	Author fullName = 'sebastiansastre' ifTrue:[
		^ '/Users/sebastian/Documents/git/flow' asPath].
	
	
	Author fullName = 'paulwilke' ifTrue:[
		^ 'C:\Users\paul\Development\GitRepo\kando\flow' asPath]  	! !

!KOAmberJSFileLibrary methodsFor: '*Flow-Announcements' stamp: 'sebastiansastre 3/13/2012 23:34'!
FlowAnnouncementsJs
	^ 'smalltalk.addPackage(''Flow-Announcements'', {});
smalltalk.addClass(''Announcement'', smalltalk.Object, [''subject'', ''isBubbling''], ''Flow-Announcements'');
smalltalk.addMethod(
unescape(''_subject''),
smalltalk.method({
selector: unescape(''subject''),
category: ''accessing'',
fn: function () {
var self=this;
return self[''@subject''];
return self;},
args: [],
source: unescape(''subject%0A%0A%09%5E%20subject''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Announcement);

smalltalk.addMethod(
unescape(''_subject_''),
smalltalk.method({
selector: unescape(''subject%3A''),
category: ''accessing'',
fn: function (anObject) {
var self=this;
(self[''@subject'']=anObject);
return self;},
args: ["anObject"],
source: unescape(''subject%3A%20anObject%0A%0A%09subject%20%3A%3D%20anObject''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Announcement);

smalltalk.addMethod(
unescape(''_initialize''),
smalltalk.method({
selector: unescape(''initialize''),
category: ''initialization'',
fn: function (){
var self=this;
smalltalk.send(self, "_initialize", [], smalltalk.Object);
(self[''@isBubbling'']=true);
return self;},
args: [],
source: unescape(''initialize%0A%0A%09super%20initialize.%0A%09%0A%09isBubbling%20%3A%3D%20true''),
messageSends: ["initialize"],
referencedClasses: []
}),
smalltalk.Announcement);

smalltalk.addMethod(
unescape(''_stop''),
smalltalk.method({
selector: unescape(''stop''),
category: ''actions'',
fn: function (){
var self=this;
(self[''@isBubbling'']=false);
return self;},
args: [],
source: unescape(''stop%0A%09%0A%09isBubbling%20%3A%3D%20false''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Announcement);

smalltalk.addMethod(
unescape(''_isBubbling''),
smalltalk.method({
selector: unescape(''isBubbling''),
category: ''testing'',
fn: function (){
var self=this;
return self[''@isBubbling''];
return self;},
args: [],
source: unescape(''isBubbling%0A%09%0A%09%5E%20isBubbling%20''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Announcement);


smalltalk.addMethod(
unescape(''_for_''),
smalltalk.method({
selector: unescape(''for%3A''),
category: ''not yet classified'',
fn: function (anObject) {
var self=this;
return (function($rec){smalltalk.send($rec, "_subject_", [anObject]);return smalltalk.send($rec, "_yourself", []);})(smalltalk.send(self, "_new", []));
return self;},
args: ["anObject"],
source: unescape(''for%3A%20anObject%0A%0A%09%5E%20self%20new%0A%09%09subject%3A%20anObject%3B%0A%09%09yourself''),
messageSends: ["subject:", "yourself", "new"],
referencedClasses: []
}),
smalltalk.Announcement.klass);


smalltalk.addClass(''AjaxSuccess'', smalltalk.Announcement, [], ''Flow-Announcements'');


smalltalk.addClass(''ModelUpdated'', smalltalk.Announcement, [], ''Flow-Announcements'');


smalltalk.addClass(''ModelCreated'', smalltalk.Announcement, [], ''Flow-Announcements'');


smalltalk.addClass(''ModelDeleted'', smalltalk.Announcement, [], ''Flow-Announcements'');


smalltalk.addClass(''ModelRefreshed'', smalltalk.Announcement, [], ''Flow-Announcements'');


'! !

!KOAmberJSFileLibrary methodsFor: '*Flow-Announcements' stamp: 'sebastiansastre 3/13/2012 23:34'!
FlowAnnouncementsdeployJs
	^ 'smalltalk.addPackage(''Flow-Announcements'', {});
smalltalk.addClass(''Announcement'', smalltalk.Object, [''subject'', ''isBubbling''], ''Flow-Announcements'');
smalltalk.addMethod(
unescape(''_subject''),
smalltalk.method({
selector: unescape(''subject''),
fn: function () {
var self=this;
return self[''@subject''];
return self;}
}),
smalltalk.Announcement);

smalltalk.addMethod(
unescape(''_subject_''),
smalltalk.method({
selector: unescape(''subject%3A''),
fn: function (anObject) {
var self=this;
(self[''@subject'']=anObject);
return self;}
}),
smalltalk.Announcement);

smalltalk.addMethod(
unescape(''_initialize''),
smalltalk.method({
selector: unescape(''initialize''),
fn: function (){
var self=this;
smalltalk.send(self, "_initialize", [], smalltalk.Object);
(self[''@isBubbling'']=true);
return self;}
}),
smalltalk.Announcement);

smalltalk.addMethod(
unescape(''_stop''),
smalltalk.method({
selector: unescape(''stop''),
fn: function (){
var self=this;
(self[''@isBubbling'']=false);
return self;}
}),
smalltalk.Announcement);

smalltalk.addMethod(
unescape(''_isBubbling''),
smalltalk.method({
selector: unescape(''isBubbling''),
fn: function (){
var self=this;
return self[''@isBubbling''];
return self;}
}),
smalltalk.Announcement);


smalltalk.addMethod(
unescape(''_for_''),
smalltalk.method({
selector: unescape(''for%3A''),
fn: function (anObject) {
var self=this;
return (function($rec){smalltalk.send($rec, "_subject_", [anObject]);return smalltalk.send($rec, "_yourself", []);})(smalltalk.send(self, "_new", []));
return self;}
}),
smalltalk.Announcement.klass);


smalltalk.addClass(''AjaxSuccess'', smalltalk.Announcement, [], ''Flow-Announcements'');


smalltalk.addClass(''ModelUpdated'', smalltalk.Announcement, [], ''Flow-Announcements'');


smalltalk.addClass(''ModelCreated'', smalltalk.Announcement, [], ''Flow-Announcements'');


smalltalk.addClass(''ModelDeleted'', smalltalk.Announcement, [], ''Flow-Announcements'');


smalltalk.addClass(''ModelRefreshed'', smalltalk.Announcement, [], ''Flow-Announcements'');


'! !

!KOAmberJSFileLibrary methodsFor: '*Flow-Errors' stamp: 'sebastiansastre 3/13/2012 23:34'!
FlowErrorsJs
	^ 'smalltalk.addPackage(''Flow-Errors'', {});
smalltalk.addClass(''FlowError'', smalltalk.Error, [], ''Flow-Errors'');


smalltalk.addClass(''ModelUpdateError'', smalltalk.FlowError, [], ''Flow-Errors'');


smalltalk.addClass(''ModelReadError'', smalltalk.FlowError, [], ''Flow-Errors'');


smalltalk.addClass(''ModelDeleteError'', smalltalk.FlowError, [], ''Flow-Errors'');


smalltalk.addClass(''ModelCreateError'', smalltalk.FlowError, [], ''Flow-Errors'');


smalltalk.addClass(''AjaxError'', smalltalk.FlowError, [], ''Flow-Errors'');


smalltalk.addClass(''ModelRefeshError'', smalltalk.FlowError, [], ''Flow-Errors'');


'! !

!KOAmberJSFileLibrary methodsFor: '*Flow-Errors' stamp: 'sebastiansastre 3/13/2012 23:34'!
FlowErrorsdeployJs
	^ 'smalltalk.addPackage(''Flow-Errors'', {});
smalltalk.addClass(''FlowError'', smalltalk.Error, [], ''Flow-Errors'');


smalltalk.addClass(''ModelUpdateError'', smalltalk.FlowError, [], ''Flow-Errors'');


smalltalk.addClass(''ModelReadError'', smalltalk.FlowError, [], ''Flow-Errors'');


smalltalk.addClass(''ModelDeleteError'', smalltalk.FlowError, [], ''Flow-Errors'');


smalltalk.addClass(''ModelCreateError'', smalltalk.FlowError, [], ''Flow-Errors'');


smalltalk.addClass(''AjaxError'', smalltalk.FlowError, [], ''Flow-Errors'');


smalltalk.addClass(''ModelRefeshError'', smalltalk.FlowError, [], ''Flow-Errors'');


'! !

!KOAmberJSFileLibrary methodsFor: '*Flow' stamp: 'sebastiansastre 3/15/2012 14:38'!
FlowJs
	^ 'smalltalk.addPackage(''Flow'', {});
smalltalk.addClass(''Model'', smalltalk.Object, [''announcer''], ''Flow'');
smalltalk.Model.comment=unescape(''Models%20are%20meant%20to%20be%20used%20as%20local-only%20%28at%20the%20DOM%29%20objects%20that%20should%20be%20transident%20in%20nature%20%28not-persistent%29'')
smalltalk.addMethod(
unescape(''_announcer''),
smalltalk.method({
selector: unescape(''announcer''),
category: ''accessing'',
fn: function () {
var self=this;
return (($receiver = self[''@announcer'']) == nil || $receiver == undefined) ? (function(){return smalltalk.send(self, "_initializeAnnouncer", []);})() : $receiver;
return self;},
args: [],
source: unescape(''announcer%0A%0A%09%5E%20announcer%20ifNil%3A%5Bself%20initializeAnnouncer%5D''),
messageSends: ["ifNil:", "initializeAnnouncer"],
referencedClasses: []
}),
smalltalk.Model);

smalltalk.addMethod(
unescape(''_initializeAnnouncer''),
smalltalk.method({
selector: unescape(''initializeAnnouncer''),
category: ''initialization'',
fn: function () {
var self=this;
return (self[''@announcer'']=smalltalk.send((smalltalk.Announcer || Announcer), "_new", []));
return self;},
args: [],
source: unescape(''initializeAnnouncer%0A%0A%09%5E%20announcer%20%3A%3D%20Announcer%20new''),
messageSends: ["new"],
referencedClasses: ["Announcer"]
}),
smalltalk.Model);

smalltalk.addMethod(
unescape(''_announce_''),
smalltalk.method({
selector: unescape(''announce%3A''),
category: ''actions'',
fn: function (anAnnouncement) {
var self=this;
smalltalk.send(smalltalk.send(self, "_announcer", []), "_announce_", [anAnnouncement]);
return self;},
args: ["anAnnouncement"],
source: unescape(''announce%3A%20anAnnouncement%0A%0A%09self%20announcer%20announce%3A%20anAnnouncement''),
messageSends: ["announce:", "announcer"],
referencedClasses: []
}),
smalltalk.Model);

smalltalk.addMethod(
unescape(''_on_do_''),
smalltalk.method({
selector: unescape(''on%3Ado%3A''),
category: ''actions'',
fn: function (anAnnouncementClass, aReactionBlock) {
var self=this;
return smalltalk.send(smalltalk.send(self, "_announcer", []), "_on_do_", [anAnnouncementClass, aReactionBlock]);
return self;},
args: ["anAnnouncementClass", "aReactionBlock"],
source: unescape(''on%3A%20anAnnouncementClass%20do%3A%20aReactionBlock%0A%0A%09%5E%20self%20announcer%20on%3A%20anAnnouncementClass%20do%3A%20aReactionBlock''),
messageSends: ["on:do:", "announcer"],
referencedClasses: []
}),
smalltalk.Model);


smalltalk.addMethod(
unescape(''_initialize''),
smalltalk.method({
selector: unescape(''initialize''),
category: ''initialization'',
fn: function () {
var self=this;
smalltalk.send((smalltalk.Package || Package), "_defaultCommitPathSt_", [unescape("/amberMP/commitCode")]);
smalltalk.send((smalltalk.Package || Package), "_defaultCommitPathJs_", [unescape("/amberMP/commitCode")]);
return self;},
args: [],
source: unescape(''initialize%0A%0A%09%22Set%20amber%20to%20commit%20using%20the%20AmberMP%27s%20API%22%0A%09Package%20defaultCommitPathSt%3A%20%27/amberMP/commitCode%27.%0A%09Package%20defaultCommitPathJs%3A%20%27/amberMP/commitCode%27%0A''),
messageSends: ["defaultCommitPathSt:", "defaultCommitPathJs:"],
referencedClasses: ["Package"]
}),
smalltalk.Model.klass);

smalltalk.addMethod(
unescape(''_basePath''),
smalltalk.method({
selector: unescape(''basePath''),
category: ''accessing'',
fn: function () {
var self=this;
return "api";
return self;},
args: [],
source: unescape(''basePath%0A%09%22Answers%20the%20base%20path%20for%20the%20application%27s%20API.%0A%09Subclasses%20should%20override%20if%20appropriate.%22%0A%0A%09%22By%20default%20we%20use%20the%20AmberMP%27s%20default%22%0A%09%5E%20%27api%27''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Model.klass);

smalltalk.addMethod(
unescape(''_path''),
smalltalk.method({
selector: unescape(''path''),
category: ''accessing'',
fn: function () {
var self=this;
return smalltalk.send(smalltalk.send(smalltalk.send(self, "_basePath", []), "__comma", [unescape("/")]), "__comma", [smalltalk.send(self, "_modelPath", [])]);
return self;},
args: [],
source: unescape(''path%0A%09%5E%20self%20basePath%2C%27/%27%2C%20self%20modelPath''),
messageSends: [unescape("%2C"), "basePath", "modelPath"],
referencedClasses: []
}),
smalltalk.Model.klass);

smalltalk.addMethod(
unescape(''_modelPath''),
smalltalk.method({
selector: unescape(''modelPath''),
category: ''accessing'',
fn: function () {
var self=this;
return smalltalk.send(self, "_name", []);
return self;},
args: [],
source: unescape(''modelPath%0A%09%22Answers%20the%20path%20for%20posting%20instances%20of%20this%20model.%0A%09Subclasses%20should%20override%20if%20appropriate.%22%0A%0A%09%5E%20self%20name''),
messageSends: ["name"],
referencedClasses: []
}),
smalltalk.Model.klass);

smalltalk.addMethod(
unescape(''_jsonIgnoreInstanceVariables''),
smalltalk.method({
selector: unescape(''jsonIgnoreInstanceVariables''),
category: ''accessing'',
fn: function (){
var self=this;
return smalltalk.send(smalltalk.send(self, "_jsonIgnoreInstanceVariables", [], smalltalk.Object.klass), "__comma", [["announcer"]]);
return self;},
args: [],
source: unescape(''jsonIgnoreInstanceVariables%0A%09%22Answers%20the%20names%20of%20the%20instance%20variables%20that%20have%20to%20be%20ignored%20when%20serializing/reifying%20this%20objects%20to/from%20json.%0A%09Subclasses%20should%20override%20when%20appropiate%20%28but%20don%27t%20forget%20concatenation%20with%20super%27s%29.%22%0A%0A%09%5E%20super%20jsonIgnoreInstanceVariables%2C%20%0A%09%23%28%27announcer%27%29''),
messageSends: [unescape("%2C"), "jsonIgnoreInstanceVariables"],
referencedClasses: []
}),
smalltalk.Model.klass);


smalltalk.addClass(''Presenter'', smalltalk.Widget, [''children'', ''model'', ''announcer'', ''parent'', ''visible'', ''wrapper''], ''Flow'');
smalltalk.addMethod(
unescape(''_announcer''),
smalltalk.method({
selector: unescape(''announcer''),
category: ''accessing'',
fn: function () {
var self=this;
return (($receiver = self[''@announcer'']) == nil || $receiver == undefined) ? (function(){return smalltalk.send(self, "_initializeAnnouncer", []);})() : $receiver;
return self;},
args: [],
source: unescape(''announcer%0A%0A%09%5E%20announcer%20ifNil%3A%5Bself%20initializeAnnouncer%5D''),
messageSends: ["ifNil:", "initializeAnnouncer"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_initializeAnnouncer''),
smalltalk.method({
selector: unescape(''initializeAnnouncer''),
category: ''initialization'',
fn: function () {
var self=this;
return (self[''@announcer'']=smalltalk.send((smalltalk.Announcer || Announcer), "_new", []));
return self;},
args: [],
source: unescape(''initializeAnnouncer%0A%0A%09%5E%20announcer%20%3A%3D%20Announcer%20new''),
messageSends: ["new"],
referencedClasses: ["Announcer"]
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_announce_''),
smalltalk.method({
selector: unescape(''announce%3A''),
category: ''accessing'',
fn: function (anAnnouncement) {
var self=this;
smalltalk.send(smalltalk.send(self, "_announcer", []), "_announce_", [anAnnouncement]);
((($receiver = smalltalk.send(anAnnouncement, "_isBubbling", [])).klass === smalltalk.Boolean) ? ($receiver ? (function(){return (($receiver = smalltalk.send(self, "_parent", [])) !!= nil && $receiver !!= undefined) ? (function(){return smalltalk.send(smalltalk.send(self, "_parent", []), "_announce_", [anAnnouncement]);})() : nil;})() : nil) : smalltalk.send($receiver, "_ifTrue_", [(function(){return (($receiver = smalltalk.send(self, "_parent", [])) !!= nil && $receiver !!= undefined) ? (function(){return smalltalk.send(smalltalk.send(self, "_parent", []), "_announce_", [anAnnouncement]);})() : nil;})]));
return self;},
args: ["anAnnouncement"],
source: unescape(''announce%3A%20anAnnouncement%0A%09%22Announce%20%28bubbling%20through%20parents%29%22%0A%09%0A%09self%20announcer%20announce%3A%20anAnnouncement.%0A%0A%09anAnnouncement%20isBubbling%20ifTrue%3A%5B%0A%09%09self%20parent%20ifNotNil%3A%5B%0A%09%09%09self%20parent%20announce%3A%20anAnnouncement%5D%5D%0A''),
messageSends: ["announce:", "announcer", "ifTrue:", "isBubbling", "ifNotNil:", "parent"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_hasParent''),
smalltalk.method({
selector: unescape(''hasParent''),
category: ''testing'',
fn: function () {
var self=this;
return smalltalk.send(self[''@parent''], "_notNil", []);
return self;},
args: [],
source: unescape(''hasParent%0A%0A%09%5E%20parent%20notNil''),
messageSends: ["notNil"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_parent''),
smalltalk.method({
selector: unescape(''parent''),
category: ''accessing'',
fn: function () {
var self=this;
return self[''@parent''];
return self;},
args: [],
source: unescape(''parent%0A%0A%09%5E%20parent''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_parent_''),
smalltalk.method({
selector: unescape(''parent%3A''),
category: ''accessing'',
fn: function (aKdWidget) {
var self=this;
(self[''@parent'']=aKdWidget);
return self;},
args: ["aKdWidget"],
source: unescape(''parent%3A%20aKdWidget%0A%0A%09parent%20%3A%3D%20aKdWidget''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_children''),
smalltalk.method({
selector: unescape(''children''),
category: ''accessing'',
fn: function () {
var self=this;
return (($receiver = self[''@children'']) == nil || $receiver == undefined) ? (function(){return smalltalk.send(self, "_initializeChildren", []);})() : $receiver;
return self;},
args: [],
source: unescape(''children%0A%0A%09%5E%20children%20ifNil%3A%5Bself%20initializeChildren%5D''),
messageSends: ["ifNil:", "initializeChildren"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_initializeChildren''),
smalltalk.method({
selector: unescape(''initializeChildren''),
category: ''initialization'',
fn: function () {
var self=this;
return (self[''@children'']=smalltalk.send((smalltalk.Dictionary || Dictionary), "_new", []));
return self;},
args: [],
source: unescape(''initializeChildren%0A%0A%09%5E%20children%20%3A%3D%20Dictionary%20new''),
messageSends: ["new"],
referencedClasses: ["Dictionary"]
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_at_ifAbsent_''),
smalltalk.method({
selector: unescape(''at%3AifAbsent%3A''),
category: ''accessing'',
fn: function (aKey, aBlock) {
var self=this;
return smalltalk.send(smalltalk.send(self, "_children", []), "_at_ifAbsent_", [aKey, aBlock]);
return self;},
args: ["aKey", "aBlock"],
source: unescape(''at%3A%20aKey%20ifAbsent%3A%20aBlock%0A%0A%09%5E%20self%20children%20at%3A%20aKey%20ifAbsent%3A%20aBlock''),
messageSends: ["at:ifAbsent:", "children"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_at_''),
smalltalk.method({
selector: unescape(''at%3A''),
category: ''accessing'',
fn: function (aKey) {
var self=this;
return smalltalk.send(smalltalk.send(self, "_children", []), "_at_", [aKey]);
return self;},
args: ["aKey"],
source: unescape(''at%3A%20aKey%0A%09%22Answers%20the%20%28sub%29widget%20stored%20in%20this%20receiver%20at%20aKey%22%0A%09%5E%20self%20children%20at%3A%20aKey''),
messageSends: ["at:", "children"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_allSubChildren''),
smalltalk.method({
selector: unescape(''allSubChildren''),
category: ''accessing'',
fn: function () {
var self=this;
return smalltalk.send(self, "_addAllSubChildrenTo_", [smalltalk.send((smalltalk.Set || Set), "_new", [])]);
return self;},
args: [],
source: unescape(''allSubChildren%0A%0A%09%5E%20self%20addAllSubChildrenTo%3A%20Set%20new''),
messageSends: ["addAllSubChildrenTo:", "new"],
referencedClasses: ["Set"]
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_addAllSubChildrenTo_''),
smalltalk.method({
selector: unescape(''addAllSubChildrenTo%3A''),
category: ''accessing'',
fn: function (aSet) {
var self=this;
var subChildren=nil;
(subChildren=smalltalk.send(smalltalk.send(self, "_children", []), "_values", []));
smalltalk.send(subChildren, "_do_", [(function(aKdWidget){return (($receiver = aKdWidget) !!= nil && $receiver !!= undefined) ? (function(){smalltalk.send(aSet, "_add_", [aKdWidget]);return smalltalk.send(aKdWidget, "_addAllSubChildrenTo_", [aSet]);})() : nil;})]);
return aSet;
return self;},
args: ["aSet"],
source: unescape(''addAllSubChildrenTo%3A%20aSet%0A%09%22Adds%20all%20the%20sub%20children%20of%20the%20receiver%20to%20aSet.%22%0A%09%0A%09%7C%20subChildren%20%7C%0A%09%0A%09subChildren%20%3A%3D%20self%20children%20values.%0A%09subChildren%20do%3A%5B%3AaKdWidget%7C%0A%09%09aKdWidget%20ifNotNil%3A%5B%0A%09%09%09aSet%20add%3A%20aKdWidget.%0A%09%09%09aKdWidget%20addAllSubChildrenTo%3A%20aSet%5D%5D.%0A%09%0A%09%5E%20aSet%0A''),
messageSends: ["values", "children", "do:", "ifNotNil:", "add:", "addAllSubChildrenTo:"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_at_put_''),
smalltalk.method({
selector: unescape(''at%3Aput%3A''),
category: ''accessing'',
fn: function (aKey, aPresenter){
var self=this;
smalltalk.send(self, "_childrenAt_put_", [aKey, aPresenter]);
return aPresenter;
return self;},
args: ["aKey", "aPresenter"],
source: unescape(''at%3A%20aKey%20put%3A%20aPresenter%20%0A%0A%09self%20childrenAt%3A%20aKey%20put%3A%20aPresenter.%0A%0A%09%5E%20aPresenter%20''),
messageSends: ["childrenAt:put:"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_childrenAt_''),
smalltalk.method({
selector: unescape(''childrenAt%3A''),
category: ''accessing'',
fn: function (aKey) {
var self=this;
return smalltalk.send(self, "_at_", [aKey]);
return self;},
args: ["aKey"],
source: unescape(''childrenAt%3A%20aKey%0A%09%22Answers%20the%20%28sub%29widget%20that%20the%20receiver%20has%20stored%20at%20aKey%22%0A%09%5E%20self%20at%3A%20aKey''),
messageSends: ["at:"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_doesNotUnderstand_''),
smalltalk.method({
selector: unescape(''doesNotUnderstand%3A''),
category: ''actions'',
fn: function (aMessage){
var self=this;
return ((($receiver = smalltalk.send(self, "_hasPresenterAt_", [smalltalk.send(smalltalk.send(aMessage, "_selector", []), "_asSymbol", [])])).klass === smalltalk.Boolean) ? ($receiver ? (function(){return smalltalk.send(self, "_at_", [smalltalk.send(smalltalk.send(aMessage, "_selector", []), "_asSymbol", [])]);})() : (function(){return smalltalk.send(self, "_doesNotUnderstand_", [aMessage], smalltalk.Widget);})()) : smalltalk.send($receiver, "_ifTrue_ifFalse_", [(function(){return smalltalk.send(self, "_at_", [smalltalk.send(smalltalk.send(aMessage, "_selector", []), "_asSymbol", [])]);}), (function(){return smalltalk.send(self, "_doesNotUnderstand_", [aMessage], smalltalk.Widget);})]));
return self;},
args: ["aMessage"],
source: unescape(''doesNotUnderstand%3A%20aMessage%0A%0A%09%5E%20%28self%20hasPresenterAt%3A%20aMessage%20selector%20asSymbol%29%0A%09%09ifTrue%3A%5Bself%20at%3A%20aMessage%20selector%20asSymbol%5D%0A%09%09ifFalse%3A%5Bsuper%20doesNotUnderstand%3A%20aMessage%5D%0A''),
messageSends: ["ifTrue:ifFalse:", "hasPresenterAt:", "asSymbol", "selector", "at:", "doesNotUnderstand:"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_model_''),
smalltalk.method({
selector: unescape(''model%3A''),
category: ''initialization'',
fn: function (aModel) {
var self=this;
smalltalk.send(self, "_setModel_", [aModel]);
smalltalk.send(self, "_onModelUpdated", []);
return self;},
args: ["aModel"],
source: unescape(''model%3A%20aModel%0A%09%22Set%20the%20model%20of%20this%20widget%22%0A%0A%09self%20setModel%3A%20aModel.%0A%09%0A%09self%20onModelUpdated''),
messageSends: ["setModel:", "onModelUpdated"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_onModelUpdated''),
smalltalk.method({
selector: unescape(''onModelUpdated''),
category: ''reactions'',
fn: function () {
var self=this;

return self;},
args: [],
source: unescape(''onModelUpdated%0A%09%22The%20model%20has%20been%20updated.%0A%09This%20is%20your%20chance%20to%20react%20about%20it.%22%0A%09%0A%09%22no-op%22''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_model''),
smalltalk.method({
selector: unescape(''model''),
category: ''accessing'',
fn: function () {
var self=this;
return self[''@model''];
return self;},
args: [],
source: unescape(''model%0A%09%0A%09%5E%20model''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_setModel_''),
smalltalk.method({
selector: unescape(''setModel%3A''),
category: ''actions'',
fn: function (aModel) {
var self=this;
smalltalk.send(self, "_onAboutToUpdateModel_", [aModel]);
(self[''@model'']=aModel);
return self;},
args: ["aModel"],
source: unescape(''setModel%3A%20aModel%0A%09%22This%20widget%20is%20going%20updating%20its%20model%20to%20aModel%22%0A%0A%09self%20onAboutToUpdateModel%3A%20aModel.%0A%0A%09model%20%3A%3D%20aModel%20%0A''),
messageSends: ["onAboutToUpdateModel:"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_onAboutToUpdateModel_''),
smalltalk.method({
selector: unescape(''onAboutToUpdateModel%3A''),
category: ''reactions'',
fn: function (aModel) {
var self=this;

return self;},
args: ["aModel"],
source: unescape(''onAboutToUpdateModel%3A%20aModel%0A%09%22The%20model%20is%20about%20to%20be%20updated.%0A%09This%20is%20your%20chance%20to%20react%20about%20it.%22%0A%09%0A%09%22no-op%22''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_open''),
smalltalk.method({
selector: unescape(''open''),
category: ''actions'',
fn: function (){
var self=this;
smalltalk.send(self, "_onAboutToOpen", []);
smalltalk.send(self, "_paintOnJQuery_", [smalltalk.send("body", "_asJQuery", [])]);
return self;},
args: [],
source: unescape(''open%0A%09%0A%09self%20onAboutToOpen.%0A%0A%09self%20paintOnJQuery%3A%20%27body%27%20asJQuery%20''),
messageSends: ["onAboutToOpen", "paintOnJQuery:", "asJQuery"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_onAboutToOpen''),
smalltalk.method({
selector: unescape(''onAboutToOpen''),
category: ''reactions'',
fn: function () {
var self=this;

return self;},
args: [],
source: unescape(''onAboutToOpen%0A%0A%09%22no-op%22''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_ifAbsentAt_put_''),
smalltalk.method({
selector: unescape(''ifAbsentAt%3Aput%3A''),
category: ''accessing'',
fn: function (aKey, aBlock){
var self=this;
return ((($receiver = smalltalk.send(self, "_hasPresenterAt_", [aKey])).klass === smalltalk.Boolean) ? (!! $receiver ? (function(){return smalltalk.send(self, "_at_put_", [aKey, smalltalk.send(aBlock, "_value", [])]);})() : nil) : smalltalk.send($receiver, "_ifFalse_", [(function(){return smalltalk.send(self, "_at_put_", [aKey, smalltalk.send(aBlock, "_value", [])]);})]));
return self;},
args: ["aKey", "aBlock"],
source: unescape(''ifAbsentAt%3A%20aKey%20put%3A%20aBlock%0A%09%22Conditionally%20%28to%20its%20abscence%29%20adds%20the%20presenter%20returned%20by%20aBlock%0A%09as%20%28sub%29widget%20of%20this%20widget%22%0A%09%0A%09%5E%20%28self%20hasPresenterAt%3A%20aKey%29%20ifFalse%3A%5B%0A%09%09self%20at%3A%20aKey%20put%3A%20aBlock%20value%5D%20''),
messageSends: ["ifFalse:", "hasPresenterAt:", "at:put:", "value"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_ifAbsentAt_put_andDo_''),
smalltalk.method({
selector: unescape(''ifAbsentAt%3Aput%3AandDo%3A''),
category: ''accessing'',
fn: function (aKey, aBlock, anotherBlock){
var self=this;
((($receiver = smalltalk.send(self, "_hasPresenterAt_", [aKey])).klass === smalltalk.Boolean) ? (!! $receiver ? (function(){smalltalk.send(self, "_at_put_", [aKey, smalltalk.send(aBlock, "_value", [])]);return smalltalk.send(anotherBlock, "_value", []);})() : nil) : smalltalk.send($receiver, "_ifFalse_", [(function(){smalltalk.send(self, "_at_put_", [aKey, smalltalk.send(aBlock, "_value", [])]);return smalltalk.send(anotherBlock, "_value", []);})]));
return self;},
args: ["aKey", "aBlock", "anotherBlock"],
source: unescape(''ifAbsentAt%3A%20aKey%20put%3A%20aBlock%20andDo%3A%20anotherBlock%0A%09%22Conditionally%20%28to%20its%20abscence%29%20adds%20the%20presenter%20answered%20by%20aBlock%0A%09as%20%28sub%29widget%20of%20this%20widget%20and%20then%20evaluates%20anotherBlock.%0A%09Note%3A%20the%20typical%20applicability%20of%20anotherBlock%20is%20the%20configuration%20%0A%09of%20the%20observation%20of%20sub%20widgets%27%20announcements%22%0A%09%0A%09%28self%20hasPresenterAt%3A%20aKey%29%20ifFalse%3A%5B%0A%09%09self%20at%3A%20aKey%20put%3A%20aBlock%20value.%0A%09%09anotherBlock%20value%5D%20''),
messageSends: ["ifFalse:", "hasPresenterAt:", "at:put:", "value"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_componentsAt_put_''),
smalltalk.method({
selector: unescape(''componentsAt%3Aput%3A''),
category: ''accessing'',
fn: function (aKey, aPresenter) {
var self=this;
smalltalk.send(smalltalk.send(self, "_children", []), "_at_put_", [aKey, aPresenter]);
((($receiver = smalltalk.send(aPresenter, "_isPresenter", [])).klass === smalltalk.Boolean) ? ($receiver ? (function(){return smalltalk.send(aPresenter, "_parent_", [self]);})() : nil) : smalltalk.send($receiver, "_ifTrue_", [(function(){return smalltalk.send(aPresenter, "_parent_", [self]);})]));
return aPresenter;
return self;},
args: ["aKey", "aPresenter"],
source: unescape(''componentsAt%3A%20aKey%20put%3A%20aPresenter%20%0A%09%22Answers%20the%20presenter%20recently%20added%20to%20the%20receiver%0A%09and%20with%20this%20receiver%20as%20its%20parent.%22%0A%09%0A%09self%20children%20at%3A%20aKey%20put%3A%20aPresenter.%0A%09aPresenter%20isPresenter%20ifTrue%3A%20%5BaPresenter%20parent%3A%20self%5D.%0A%09%5E%20aPresenter%20''),
messageSends: ["at:put:", "children", "ifTrue:", "isPresenter", "parent:"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_childrenAt_put_''),
smalltalk.method({
selector: unescape(''childrenAt%3Aput%3A''),
category: ''accessing'',
fn: function (aKey, aPresenter) {
var self=this;
smalltalk.send(smalltalk.send(self, "_children", []), "_at_put_", [aKey, aPresenter]);
((($receiver = smalltalk.send(aPresenter, "_isPresenter", [])).klass === smalltalk.Boolean) ? ($receiver ? (function(){return smalltalk.send(aPresenter, "_parent_", [self]);})() : nil) : smalltalk.send($receiver, "_ifTrue_", [(function(){return smalltalk.send(aPresenter, "_parent_", [self]);})]));
return aPresenter;
return self;},
args: ["aKey", "aPresenter"],
source: unescape(''childrenAt%3A%20aKey%20put%3A%20aPresenter%20%0A%09%22Answers%20the%20presenter%20recently%20added%20to%20the%20receiver%0A%09%28with%20this%20receiver%20as%20its%20parent%29.%22%0A%09%0A%09self%20children%20at%3A%20aKey%20put%3A%20aPresenter.%0A%09aPresenter%20isPresenter%20ifTrue%3A%20%5BaPresenter%20parent%3A%20self%5D.%0A%09%5E%20aPresenter%20''),
messageSends: ["at:put:", "children", "ifTrue:", "isPresenter", "parent:"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_on_do_''),
smalltalk.method({
selector: unescape(''on%3Ado%3A''),
category: ''accessing'',
fn: function (anAnnouncementClass, aReactionBlock) {
var self=this;
return smalltalk.send(smalltalk.send(self, "_announcer", []), "_on_do_", [anAnnouncementClass, aReactionBlock]);
return self;},
args: ["anAnnouncementClass", "aReactionBlock"],
source: unescape(''on%3A%20anAnnouncementClass%20do%3A%20aReactionBlock%0A%0A%09%5E%20self%20announcer%20on%3A%20anAnnouncementClass%20do%3A%20aReactionBlock''),
messageSends: ["on:do:", "announcer"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_newCanvasOn_''),
smalltalk.method({
selector: unescape(''newCanvasOn%3A''),
category: ''actions'',
fn: function (aJQuery) {
var self=this;
return smalltalk.send((smalltalk.HTMLCanvas || HTMLCanvas), "_onJQuery_", [aJQuery]);
return self;},
args: ["aJQuery"],
source: unescape(''newCanvasOn%3A%20aJQuery%0A%09%22Answers%20a%20new%20canvas%20based%20on%20the%20given%20aJQuery%20element.%22%0A%0A%09%5E%20HTMLCanvas%20onJQuery%3A%20aJQuery''),
messageSends: ["onJQuery:"],
referencedClasses: ["HTMLCanvas"]
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_isPresenter''),
smalltalk.method({
selector: unescape(''isPresenter''),
category: ''testing'',
fn: function () {
var self=this;
return true;
return self;},
args: [],
source: unescape(''isPresenter%0A%0A%09%5E%20true''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_visible''),
smalltalk.method({
selector: unescape(''visible''),
category: ''accessing'',
fn: function (){
var self=this;
return (($receiver = self[''@visible'']) == nil || $receiver == undefined) ? (function(){return smalltalk.send(self, "_initializeVisible", []);})() : $receiver;
return self;},
args: [],
source: unescape(''visible%0A%09%22Answers%20true%20if%20this%20presenter%20should%20be%20visible%20%0A%09as%20soon%20as%20it%20get%27s%20rendered%20%28or%20not%2C%20in%20which%20case%2C%0A%09%20it%20will%20be%20rendered%20while%20staying%20hidden%29.%22%0A%09%5E%20visible%20ifNil%3A%5Bself%20initializeVisible%5D''),
messageSends: ["ifNil:", "initializeVisible"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_visible_''),
smalltalk.method({
selector: unescape(''visible%3A''),
category: ''accessing'',
fn: function (aBoolean){
var self=this;
(self[''@visible'']=aBoolean);
return self;},
args: ["aBoolean"],
source: unescape(''visible%3A%20aBoolean%0A%09%22Sets%20if%20this%20presenter%20should%20be%20visible%20%0A%09as%20soon%20as%20it%20get%27s%20rendered%20%28or%20not%2C%20in%20which%20case%2C%0A%09%20it%20will%20be%20rendered%20while%20staying%20hidden%29.%22%0A%09visible%20%3A%3D%20aBoolean%0A''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_initializeVisible''),
smalltalk.method({
selector: unescape(''initializeVisible''),
category: ''initialization'',
fn: function (){
var self=this;
return (self[''@visible'']=false);
return self;},
args: [],
source: unescape(''initializeVisible%0A%09%22We%20set%20the%20visible%20default%20in%20false%20so%20the%20application%27s%0A%09behavior%20can%20decide%20elsewere%20how%20%28and%20if%29%20this%20presenter%20should%20appear.%22%0A%0A%09%5E%20visible%20%3A%3D%20false''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_renderOn_''),
smalltalk.method({
selector: unescape(''renderOn%3A''),
category: ''rendering'',
fn: function (html){
var self=this;
(self[''@painted'']=true);
(self[''@wrapper'']=smalltalk.send(html, "_div", []));
(function($rec){smalltalk.send($rec, "_class_", [smalltalk.send(smalltalk.send(self, "_class", []), "_name", [])]);return smalltalk.send($rec, "_with_", [(function(){return smalltalk.send(self, "_paintOn_", [html]);})]);})(self[''@wrapper'']);
return self;},
args: ["html"],
source: unescape(''renderOn%3A%20html%0A%09%22Render%20this%20presenter.%0A%09Note%3A%20presenters%20are%20always%20wrapped%20in%20a%20div%20using%20as%20CSS%20class%0A%09it%27s%20class%20name.%22%0A%0A%09painted%20%3A%3D%20true.%0A%0A%09wrapper%20%3A%3D%20html%20div.%0A%09%0A%09wrapper%0A%09%09class%3A%20self%20class%20name%3B%0A%09%09with%3A%5Bself%20paintOn%3A%20html%5D''),
messageSends: ["div", "class:", "name", "class", "with:", "paintOn:"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_hide''),
smalltalk.method({
selector: unescape(''hide''),
category: ''actions'',
fn: function (){
var self=this;
smalltalk.send(smalltalk.send(self[''@wrapper''], "_asJQuery", []), "_hide", []);
return self;},
args: [],
source: unescape(''hide%0A%0A%09wrapper%20asJQuery%20hide''),
messageSends: ["hide", "asJQuery"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_show''),
smalltalk.method({
selector: unescape(''show''),
category: ''actions'',
fn: function (){
var self=this;
smalltalk.send(smalltalk.send(self[''@wrapper''], "_asJQuery", []), "_show", []);
return self;},
args: [],
source: unescape(''show%0A%0A%09wrapper%20asJQuery%20show''),
messageSends: ["show", "asJQuery"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_asJQuery''),
smalltalk.method({
selector: unescape(''asJQuery''),
category: ''accessing'',
fn: function (){
var self=this;
return smalltalk.send(self[''@wrapper''], "_asJQuery", []);
return self;},
args: [],
source: unescape(''asJQuery%0A%09%0A%09%5E%20wrapper%20asJQuery%0A''),
messageSends: ["asJQuery"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_append_''),
smalltalk.method({
selector: unescape(''append%3A''),
category: ''actions'',
fn: function (aPresenter){
var self=this;
smalltalk.send(self[''@wrapper''], "_append_", [smalltalk.send(aPresenter, "_asJQuery", [])]);
return self;},
args: ["aPresenter"],
source: unescape(''append%3A%20aPresenter%0A%09%22Appends%20aPresenter%20to%20this%20presenter.%0A%09Note%3A%20it%27s%20actually%20appended%20to%20the%20wrapper%20of%20this%20guy.%22%0A%0A%09wrapper%20append%3A%20aPresenter%20asJQuery%0A''),
messageSends: ["append:", "asJQuery"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_paintOn_''),
smalltalk.method({
selector: unescape(''paintOn%3A''),
category: ''rendering'',
fn: function (html){
var self=this;
self;
return self;},
args: ["html"],
source: unescape(''paintOn%3A%20html%0A%09%22Paints%20the%20content%20%28the%20wrapped%20content%2C%20like%20wrapper%20NOT%20included%29%0A%09of%20this%20presenter.%20Sub%20classes%20should%20override%20as%20appropiate.%22%0A%09%0A%09%22default%20is%20paint%20nothing%22%0A%09self''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_paintOnJQuery_''),
smalltalk.method({
selector: unescape(''paintOnJQuery%3A''),
category: ''rendering'',
fn: function (aJQuery){
var self=this;
smalltalk.send(self, "_appendToJQuery_", [aJQuery]);
return self;},
args: ["aJQuery"],
source: unescape(''paintOnJQuery%3A%20aJQuery%0A%09%22Appends%20what%27s%20being%20paint%20by%20this%20presenter%20to%20aJQuery.%22%0A%0A%09self%20appendToJQuery%3A%20aJQuery''),
messageSends: ["appendToJQuery:"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_hasPresenterAt_''),
smalltalk.method({
selector: unescape(''hasPresenterAt%3A''),
category: ''testing'',
fn: function (aKey){
var self=this;
return smalltalk.send(smalltalk.send(self, "_children", []), "_includesKey_", [aKey]);
return self;},
args: ["aKey"],
source: unescape(''hasPresenterAt%3A%20aKey%0A%0A%09%5E%20self%20children%20includesKey%3A%20aKey%20''),
messageSends: ["includesKey:", "children"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_paint_''),
smalltalk.method({
selector: unescape(''paint%3A''),
category: ''actions'',
fn: function (aPresenter){
var self=this;
smalltalk.send(aPresenter, "_paintOnJQuery_", [smalltalk.send(self, "_asJQuery", [])]);
return self;},
args: ["aPresenter"],
source: unescape(''paint%3A%20aPresenter%0A%09%22Paints%20the%20%28sub%29preenter%20aPresenter%20in%20this%20%28parent%29presenter.%22%0A%0A%09aPresenter%20paintOnJQuery%3A%20self%20asJQuery%0A''),
messageSends: ["paintOnJQuery:", "asJQuery"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_hideAll''),
smalltalk.method({
selector: unescape(''hideAll''),
category: ''actions'',
fn: function (){
var self=this;
smalltalk.send(smalltalk.send(smalltalk.send(self, "_children", []), "_keys", []), "_do_", [(function(each){var child=nil;
(child=smalltalk.send(self, "_at_ifAbsent_", [each, (function(){return nil;})]));return (($receiver = child) !!= nil && $receiver !!= undefined) ? (function(){return smalltalk.send(child, "_hide", []);})() : nil;})]);
return self;},
args: [],
source: unescape(''hideAll%0A%09%22Hide%20all%20the%20children.%0A%09Note%3A%20it%27s%20done%20in%20such%20a%20way%20that%20if%20they%20don%27t%20exist%20they%20won%27t%20evoked.%22%0A%0A%09self%20children%20keys%20do%3A%5B%3Aeach%7C%20%7Cchild%7C%0A%09%09child%20%3A%3D%20self%20at%3A%20each%20ifAbsent%3A%5Bnil%5D.%0A%09%09child%20ifNotNil%3A%5B%0A%09%09%09child%20hide%5D%5D.%0A''),
messageSends: ["do:", "keys", "children", "at:ifAbsent:", "ifNotNil:", "hide"],
referencedClasses: []
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_wrapper''),
smalltalk.method({
selector: unescape(''wrapper''),
category: ''accessing'',
fn: function (){
var self=this;
return self[''@wrapper''];
return self;},
args: [],
source: unescape(''wrapper%0A%0A%09%5E%20wrapper''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter);


smalltalk.addMethod(
unescape(''_for_''),
smalltalk.method({
selector: unescape(''for%3A''),
category: ''actions'',
fn: function (aModel) {
var self=this;
return (function($rec){smalltalk.send($rec, "_model_", [aModel]);return smalltalk.send($rec, "_yourself", []);})(smalltalk.send(self, "_new", []));
return self;},
args: ["aModel"],
source: unescape(''for%3A%20aModel%0A%0A%09%5E%20self%20new%0A%09%09model%3A%20aModel%3B%0A%09%09yourself''),
messageSends: ["model:", "yourself", "new"],
referencedClasses: []
}),
smalltalk.Presenter.klass);

smalltalk.addMethod(
unescape(''_open''),
smalltalk.method({
selector: unescape(''open''),
category: ''actions'',
fn: function () {
var self=this;
return smalltalk.send(smalltalk.send(self, "_for_", [smalltalk.send(self, "_defaultModel", [])]), "_open", []);
return self;},
args: [],
source: unescape(''open%0A%0A%09%5E%20%28self%20for%3A%20self%20defaultModel%29%20open''),
messageSends: ["open", "for:", "defaultModel"],
referencedClasses: []
}),
smalltalk.Presenter.klass);

smalltalk.addMethod(
unescape(''_defaultModel''),
smalltalk.method({
selector: unescape(''defaultModel''),
category: ''accessing'',
fn: function () {
var self=this;
return nil;
return self;},
args: [],
source: unescape(''defaultModel%0A%0A%09%5E%20nil''),
messageSends: [],
referencedClasses: []
}),
smalltalk.Presenter.klass);


smalltalk.addClass(''PersistentModel'', smalltalk.Model, [''id''], ''Flow'');
smalltalk.PersistentModel.comment=unescape(''PersistentModels%20are%20objects%20that%20are%20manipulable%20here%20but%20they%27re%20also%20stored%20server%20side.'')
smalltalk.addMethod(
unescape(''_id_''),
smalltalk.method({
selector: unescape(''id%3A''),
category: ''accessing'',
fn: function (aString) {
var self=this;
(self[''@id'']=aString);
return self;},
args: ["aString"],
source: unescape(''id%3A%20aString%0A%0A%09id%20%3A%3D%20aString''),
messageSends: [],
referencedClasses: []
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_id''),
smalltalk.method({
selector: unescape(''id''),
category: ''accessing'',
fn: function () {
var self=this;
return self[''@id''];
return self;},
args: [],
source: unescape(''id%0A%0A%09%5E%20id%20''),
messageSends: [],
referencedClasses: []
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_url''),
smalltalk.method({
selector: unescape(''url''),
category: ''accessing'',
fn: function () {
var self=this;
return smalltalk.send(smalltalk.send(smalltalk.send(self, "_path", []), "__comma", [unescape("/")]), "__comma", [smalltalk.send(self[''@id''], "_asString", [])]);
return self;},
args: [],
source: unescape(''url%0A%0A%09%5E%20self%20path%2C%27/%27%2Cid%20asString%0A''),
messageSends: [unescape("%2C"), "path", "asString"],
referencedClasses: []
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onAfterCreate_''),
smalltalk.method({
selector: unescape(''onAfterCreate%3A''),
category: ''reactions'',
fn: function (x) {
var self=this;
smalltalk.send(self, "_syncWith_", [smalltalk.send(smalltalk.send(self, "_class", []), "_reify_", [x])]);
smalltalk.send(self, "_announce_", [smalltalk.send((smalltalk.ModelCreated || ModelCreated), "_for_", [self])]);
return self;},
args: ["x"],
source: unescape(''onAfterCreate%3A%20x%0A%0A%09self%20syncWith%3A%20%28self%20class%20reify%3A%20x%29.%0A%09self%20announce%3A%20%28ModelCreated%20for%3A%20self%29%0A''),
messageSends: ["syncWith:", "reify:", "class", "announce:", "for:"],
referencedClasses: ["ModelCreated"]
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_update''),
smalltalk.method({
selector: unescape(''update''),
category: ''actions'',
fn: function () {
var self=this;
smalltalk.send((typeof jQuery == ''undefined'' ? nil : jQuery), "_ajax_", [smalltalk.HashedCollection._fromPairs_([smalltalk.send(smalltalk.send(smalltalk.send("url", "__minus_gt", [smalltalk.send(self, "_path", [])]), "__comma", [unescape("/")]), "__comma", [smalltalk.send(self[''@id''], "_asString", [])]),smalltalk.send("type", "__minus_gt", ["PUT"]),smalltalk.send("data", "__minus_gt", [smalltalk.send(self, "_asJSONString", [])]),smalltalk.send("success", "__minus_gt", [(function(x){return smalltalk.send(self, "_onAfterUpdate_", [x]);})]),smalltalk.send("fail", "__minus_gt", [(function(x){return smalltalk.send(self, "_onUpdateFail_", [x]);})]),smalltalk.send("error", "__minus_gt", [(function(x){return smalltalk.send(self, "_onUpdateFail_", [x]);})])])]);
return self;},
args: [],
source: unescape(''update%0A%0A%09jQuery%20ajax%3A%20%23%7B%20%0A%09%09%27url%27%20-%3E%20self%20path%2C%20%27/%27%2Cid%20asString.%0A%09%09%27type%27%20-%3E%20%27PUT%27.%0A%09%09%27data%27%20-%3E%20self%20asJSONString.%0A%09%09%27success%27%20-%3E%20%5B%3Ax%7C%20self%20onAfterUpdate%3A%20x%5D.%0A%09%09%27fail%27%20-%3E%20%5B%3Ax%7C%20self%20onUpdateFail%3A%20x%5D.%0A%09%09%27error%27%20-%3E%20%5B%3Ax%7C%20self%20onUpdateFail%3A%20x%5D%7D%20%20%0A%0A%0A''),
messageSends: ["ajax:", unescape("%2C"), unescape("-%3E"), "path", "asString", "asJSONString", "onAfterUpdate:", "onUpdateFail:"],
referencedClasses: []
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_delete''),
smalltalk.method({
selector: unescape(''delete''),
category: ''actions'',
fn: function () {
var self=this;
smalltalk.send((typeof jQuery == ''undefined'' ? nil : jQuery), "_ajax_", [smalltalk.HashedCollection._fromPairs_([smalltalk.send("url", "__minus_gt", [smalltalk.send(smalltalk.send(smalltalk.send(self, "_path", []), "__comma", [unescape("/")]), "__comma", [smalltalk.send(self[''@id''], "_asString", [])])]),smalltalk.send("type", "__minus_gt", ["DELETE"]),smalltalk.send("success", "__minus_gt", [(function(x){return smalltalk.send(self, "_onAfterDelete_", [x]);})]),smalltalk.send("fail", "__minus_gt", [(function(x){return smalltalk.send(self, "_onDeleteFail_", [x]);})]),smalltalk.send("error", "__minus_gt", [(function(x){return smalltalk.send(self, "_onDeleteFail_", [x]);})])])]);
return self;},
args: [],
source: unescape(''delete%0A%0A%09jQuery%20ajax%3A%20%23%7B%20%0A%09%09%27url%27%20-%3E%20%28self%20path%2C%20%27/%27%2Cid%20asString%29.%0A%09%09%27type%27%20-%3E%20%27DELETE%27.%0A%09%09%27success%27%20-%3E%20%5B%3Ax%7C%20self%20onAfterDelete%3A%20x%5D.%0A%09%09%27fail%27%20-%3E%20%5B%3Ax%7C%20self%20onDeleteFail%3A%20x%5D.%0A%09%09%27error%27%20-%3E%20%5B%3Ax%7C%20self%20onDeleteFail%3A%20x%5D%7D%20%20%0A%0A''),
messageSends: ["ajax:", unescape("-%3E"), unescape("%2C"), "path", "asString", "onAfterDelete:", "onDeleteFail:"],
referencedClasses: []
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onAfterUpdate_''),
smalltalk.method({
selector: unescape(''onAfterUpdate%3A''),
category: ''reactions'',
fn: function (x) {
var self=this;
smalltalk.send(self, "_announce_", [smalltalk.send((smalltalk.ModelUpdated || ModelUpdated), "_for_", [self])]);
return self;},
args: ["x"],
source: unescape(''onAfterUpdate%3A%20x%0A%09%0A%09self%20announce%3A%20%28ModelUpdated%20for%3A%20self%29%0A''),
messageSends: ["announce:", "for:"],
referencedClasses: ["ModelUpdated"]
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onUpdateFail_''),
smalltalk.method({
selector: unescape(''onUpdateFail%3A''),
category: ''reactions'',
fn: function (x) {
var self=this;
smalltalk.send((smalltalk.ModelUpdateError || ModelUpdateError), "_signal_", [smalltalk.send("Could not update ", "__comma", [smalltalk.send(smalltalk.send(self, "_class", []), "_name", [])])]);
return self;},
args: ["x"],
source: unescape(''onUpdateFail%3A%20x%0A%0A%09ModelUpdateError%20signal%3A%20%27Could%20not%20update%20%27%2C%20self%20class%20name%09%0A''),
messageSends: ["signal:", unescape("%2C"), "name", "class"],
referencedClasses: ["ModelUpdateError"]
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onDeleteFail_''),
smalltalk.method({
selector: unescape(''onDeleteFail%3A''),
category: ''reactions'',
fn: function (x){
var self=this;
smalltalk.send((smalltalk.ModelUpdateError || ModelUpdateError), "_signal_", [smalltalk.send("Could not delete ", "__comma", [smalltalk.send(smalltalk.send(self, "_class", []), "_name", [])])]);
return self;},
args: ["x"],
source: unescape(''onDeleteFail%3A%20x%0A%0A%09ModelUpdateError%20signal%3A%20%27Could%20not%20delete%20%27%2C%20self%20class%20name%09%0A''),
messageSends: ["signal:", unescape("%2C"), "name", "class"],
referencedClasses: ["ModelUpdateError"]
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onAfterDelete_''),
smalltalk.method({
selector: unescape(''onAfterDelete%3A''),
category: ''reactions'',
fn: function (x) {
var self=this;
smalltalk.send(self, "_announce_", [smalltalk.send((smalltalk.ModelDeleted || ModelDeleted), "_for_", [self])]);
return self;},
args: ["x"],
source: unescape(''onAfterDelete%3A%20x%0A%09%0A%09self%20announce%3A%20%28ModelDeleted%20for%3A%20self%29%0A%20''),
messageSends: ["announce:", "for:"],
referencedClasses: ["ModelDeleted"]
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_path''),
smalltalk.method({
selector: unescape(''path''),
category: ''accessing'',
fn: function () {
var self=this;
return smalltalk.send(smalltalk.send(self, "_class", []), "_path", []);
return self;},
args: [],
source: unescape(''path%20%0A%0A%09%5E%20self%20class%20path''),
messageSends: ["path", "class"],
referencedClasses: []
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_syncWith_''),
smalltalk.method({
selector: unescape(''syncWith%3A''),
category: ''actions'',
fn: function (aReified){
var self=this;
var toSynchronize=nil;
(toSynchronize=smalltalk.send(smalltalk.send(smalltalk.send(self, "_class", []), "_allInstanceVariableNames", []), "_reject_", [(function(e){return smalltalk.send(smalltalk.send(smalltalk.send(self, "_class", []), "_jsonIgnoreInstanceVariables", []), "_includes_", [e]);})]));
smalltalk.send(toSynchronize, "_do_", [(function(instVarName){return smalltalk.send(self, "_instVarAt_put_", [instVarName, smalltalk.send(aReified, "_at_", [instVarName])]);})]);
return self;},
args: ["aReified"],
source: unescape(''syncWith%3A%20aReified%0A%09%22Sync%20the%20current%20values%20in%20this%20model%20%0A%09with%20the%20ones%20coming%20in%20aReifyed.%22%0A%09%7C%20toSynchronize%20%7C%0A%0A%09toSynchronize%20%3A%3D%20self%20class%20allInstanceVariableNames%20reject%3A%5B%3Ae%7C%20self%20class%20jsonIgnoreInstanceVariables%20includes%3A%20e%5D.%0A%09toSynchronize%20do%3A%5B%3AinstVarName%7C%0A%09%09self%20%0A%09%09%09instVarAt%3A%20instVarName%20%0A%09%09%09put%3A%20%28aReified%20at%3A%20instVarName%29%5D.%0A''),
messageSends: ["reject:", "allInstanceVariableNames", "class", "includes:", "jsonIgnoreInstanceVariables", "do:", "instVarAt:put:", "at:"],
referencedClasses: []
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_refresh''),
smalltalk.method({
selector: unescape(''refresh''),
category: ''actions'',
fn: function () {
var self=this;
smalltalk.send((typeof jQuery == ''undefined'' ? nil : jQuery), "_ajax_", [smalltalk.HashedCollection._fromPairs_([smalltalk.send("url", "__minus_gt", [smalltalk.send(smalltalk.send(smalltalk.send(self, "_path", []), "__comma", [unescape("/")]), "__comma", [smalltalk.send(self[''@id''], "_asString", [])])]),smalltalk.send("type", "__minus_gt", ["GET"]),smalltalk.send("success", "__minus_gt", [(function(x){return smalltalk.send(self, "_onAfterRefresh_", [x]);})]),smalltalk.send("fail", "__minus_gt", [(function(x){return smalltalk.send(self, "_onRefeshFail_", [x]);})]),smalltalk.send("error", "__minus_gt", [(function(x){return smalltalk.send(self, "_onRefreshFail_", [x]);})])])]);
return self;},
args: [],
source: unescape(''refresh%0A%09%22Re-read%20this%20model%27s%20state.%22%0A%0A%09jQuery%20ajax%3A%20%23%7B%20%0A%09%09%27url%27%20-%3E%20%28self%20path%2C%20%27/%27%2Cid%20asString%29.%0A%09%09%27type%27%20-%3E%20%27GET%27.%0A%09%09%27success%27%20-%3E%20%5B%3Ax%7C%20self%20onAfterRefresh%3A%20x%5D.%0A%09%09%27fail%27%20-%3E%20%5B%3Ax%7C%20self%20onRefeshFail%3A%20x%5D.%0A%09%09%27error%27%20-%3E%20%5B%3Ax%7C%20self%20onRefreshFail%3A%20x%5D%7D%09%09''),
messageSends: ["ajax:", unescape("-%3E"), unescape("%2C"), "path", "asString", "onAfterRefresh:", "onRefeshFail:", "onRefreshFail:"],
referencedClasses: []
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onRefreshFail_''),
smalltalk.method({
selector: unescape(''onRefreshFail%3A''),
category: ''reactions'',
fn: function (x) {
var self=this;
smalltalk.send((smalltalk.ModelRefreshError || ModelRefreshError), "_signal_", [smalltalk.send("Could not refresh ", "__comma", [smalltalk.send(smalltalk.send(self, "_class", []), "_name", [])])]);
return self;},
args: ["x"],
source: unescape(''onRefreshFail%3A%20x%0A%0A%09ModelRefreshError%20signal%3A%20%27Could%20not%20refresh%20%27%2C%20self%20class%20name%09%0A''),
messageSends: ["signal:", unescape("%2C"), "name", "class"],
referencedClasses: ["ModelRefreshError"]
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onAfterRefresh_''),
smalltalk.method({
selector: unescape(''onAfterRefresh%3A''),
category: ''reactions'',
fn: function (x) {
var self=this;
smalltalk.send(self, "_syncWith_", [smalltalk.send(smalltalk.send(self, "_class", []), "_reify_", [x])]);
smalltalk.send(self, "_announce_", [smalltalk.send((smalltalk.ModelRefreshed || ModelRefreshed), "_for_", [self])]);
return self;},
args: ["x"],
source: unescape(''onAfterRefresh%3A%20x%0A%0A%09self%20syncWith%3A%20%28self%20class%20reify%3A%20x%29.%0A%09self%20announce%3A%20%28ModelRefreshed%20for%3A%20self%29%0A''),
messageSends: ["syncWith:", "reify:", "class", "announce:", "for:"],
referencedClasses: ["ModelRefreshed"]
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_save''),
smalltalk.method({
selector: unescape(''save''),
category: ''actions'',
fn: function () {
var self=this;
return smalltalk.send(self, "_update", []);
return self;},
args: [],
source: unescape(''save%0A%09%5E%20self%20update''),
messageSends: ["update"],
referencedClasses: []
}),
smalltalk.PersistentModel);


smalltalk.addMethod(
unescape(''_atId_''),
smalltalk.method({
selector: unescape(''atId%3A''),
category: ''accessing'',
fn: function (anId) {
var self=this;
return smalltalk.send(self, "_read_", [anId]);
return self;},
args: ["anId"],
source: unescape(''atId%3A%20anId%0A%09%22Answers%20the%20instance%20of%20this%20model%20found%20at%20anId%20%28or%20nil%29%22%0A%20%0A%09%5E%20self%20read%3A%20anId%0A''),
messageSends: ["read:"],
referencedClasses: []
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_read_''),
smalltalk.method({
selector: unescape(''read%3A''),
category: ''actions'',
fn: function (anId) {
var self=this;
smalltalk.send((typeof jQuery == ''undefined'' ? nil : jQuery), "_ajax_", [smalltalk.HashedCollection._fromPairs_([smalltalk.send("url", "__minus_gt", [smalltalk.send(smalltalk.send(smalltalk.send(self, "_path", []), "__comma", [unescape("/")]), "__comma", [anId])]),smalltalk.send("type", "__minus_gt", ["GET"]),smalltalk.send("sucess", "__minus_gt", [(function(x){return smalltalk.send(self, "_onAfterRead_", [x]);})]),smalltalk.send("fail", "__minus_gt", [(function(x){return smalltalk.send(self, "_onReadFail_", [x]);})]),smalltalk.send("error", "__minus_gt", [(function(x){return smalltalk.send(self, "_onReadFail_", [x]);})])])]);
return self;},
args: ["anId"],
source: unescape(''read%3A%20anId%0A%0A%09jQuery%20ajax%3A%20%23%7B%20%0A%09%09%27url%27%20-%3E%20%28self%20path%2C%20%27/%27%2CanId%29.%0A%09%09%27type%27%20-%3E%20%27GET%27.%0A%09%09%27sucess%27%20-%3E%20%5B%3Ax%7C%20self%20onAfterRead%3A%20x%5D.%0A%09%09%27fail%27%20-%3E%20%5B%3Ax%7C%20self%20onReadFail%3A%20x%5D.%0A%09%09%27error%27%20-%3E%20%5B%3Ax%7C%20self%20onReadFail%3A%20x%5D%7D''),
messageSends: ["ajax:", unescape("-%3E"), unescape("%2C"), "path", "onAfterRead:", "onReadFail:"],
referencedClasses: []
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_create''),
smalltalk.method({
selector: unescape(''create''),
category: ''actions'',
fn: function (){
var self=this;
var newInstance=nil;
(newInstance=smalltalk.send(smalltalk.send(self, "_basicNew", []), "_initialize", []));
smalltalk.send((typeof jQuery == ''undefined'' ? nil : jQuery), "_ajax_", [smalltalk.HashedCollection._fromPairs_([smalltalk.send("url", "__minus_gt", [smalltalk.send(self, "_path", [])]),smalltalk.send("type", "__minus_gt", ["POST"]),smalltalk.send("success", "__minus_gt", [(function(x){return smalltalk.send(newInstance, "_onAfterCreate_", [x]);})]),smalltalk.send("fail", "__minus_gt", [(function(x){return smalltalk.send(self, "_onCreateFail_", [x]);})]),smalltalk.send("error", "__minus_gt", [(function(x){return smalltalk.send(self, "_onCreateFail_", [x]);})])])]);
return newInstance;
return self;},
args: [],
source: unescape(''create%0A%0A%09%7C%20newInstance%20%7C%0A%0A%09newInstance%20%3A%3D%20self%20basicNew%20initialize.%0A%0A%09jQuery%20ajax%3A%20%23%7B%20%0A%09%09%27url%27%20-%3E%20self%20path.%0A%09%09%27type%27%20-%3E%20%27POST%27.%0A%09%09%27success%27%20-%3E%20%5B%3Ax%7C%20newInstance%20onAfterCreate%3A%20x%5D.%0A%09%09%27fail%27%20-%3E%20%5B%3Ax%7C%20self%20onCreateFail%3A%20x%5D.%0A%09%09%27error%27%20-%3E%20%5B%3Ax%7C%20self%20onCreateFail%3A%20x%5D%7D.%0A%0A%09%5E%20newInstance%0A''),
messageSends: ["initialize", "basicNew", "ajax:", unescape("-%3E"), "path", "onAfterCreate:", "onCreateFail:"],
referencedClasses: []
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_onCreateFail_''),
smalltalk.method({
selector: unescape(''onCreateFail%3A''),
category: ''reactions'',
fn: function (x) {
var self=this;
smalltalk.send((smalltalk.ModelCreateError || ModelCreateError), "_signal_", [smalltalk.send(smalltalk.send(smalltalk.send("Could not create ", "__comma", [smalltalk.send(self, "_name", [])]), "__comma", [":  "]), "__comma", [smalltalk.send(x, "_responseText", [])])]);
return self;},
args: ["x"],
source: unescape(''onCreateFail%3A%20x%0A%0A%09ModelCreateError%20signal%3A%20%27Could%20not%20create%20%27%2C%20self%20name%2C%27%3A%20%20%27%2C%20x%20responseText%0A''),
messageSends: ["signal:", unescape("%2C"), "name", "responseText"],
referencedClasses: ["ModelCreateError"]
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_onReadFail_''),
smalltalk.method({
selector: unescape(''onReadFail%3A''),
category: ''reactions'',
fn: function (x) {
var self=this;
smalltalk.send((smalltalk.ModelReadError || ModelReadError), "_signal_", [smalltalk.send(smalltalk.send(smalltalk.send("Could not read ", "__comma", [smalltalk.send(self, "_name", [])]), "__comma", [":  "]), "__comma", [smalltalk.send(x, "_responseText", [])])]);
return self;},
args: ["x"],
source: unescape(''onReadFail%3A%20x%0A%0A%09ModelReadError%20signal%3A%20%27Could%20not%20read%20%27%2C%20self%20name%2C%27%3A%20%20%27%2C%20x%20responseText%0A''),
messageSends: ["signal:", unescape("%2C"), "name", "responseText"],
referencedClasses: ["ModelReadError"]
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_onAfterRead_''),
smalltalk.method({
selector: unescape(''onAfterRead%3A''),
category: ''reactions'',
fn: function (x) {
var self=this;
return smalltalk.send(self, "_reify_", [x]);
return self;},
args: ["x"],
source: unescape(''onAfterRead%3A%20x%0A%0A%09%5E%20self%20reify%3A%20x''),
messageSends: ["reify:"],
referencedClasses: []
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_reify_''),
smalltalk.method({
selector: unescape(''reify%3A''),
category: ''actions'',
fn: function (aJSONString) {
var self=this;
return smalltalk.send((smalltalk.JSON || JSON), "_parse_", [aJSONString]);
return self;},
args: ["aJSONString"],
source: unescape(''reify%3A%20aJSONString%0A%09%22Returns%20a%20simple%20javascript%20object%20with%0A%09the%20attributes%20meant%20for%20the%20matching%20instance%20%0A%09of%20this%20model.%22%0A%0A%09%5E%20JSON%20parse%3A%20aJSONString''),
messageSends: ["parse:"],
referencedClasses: ["JSON"]
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_new''),
smalltalk.method({
selector: unescape(''new''),
category: ''actions'',
fn: function () {
var self=this;
return smalltalk.send(self, "_create", []);
return self;},
args: [],
source: unescape(''new%0A%09%0A%09%5E%20self%20create''),
messageSends: ["create"],
referencedClasses: []
}),
smalltalk.PersistentModel.klass);


'! !

!KOAmberJSFileLibrary methodsFor: '*Flow' stamp: 'sebastiansastre 3/15/2012 14:38'!
FlowdeployJs
	^ 'smalltalk.addPackage(''Flow'', {});
smalltalk.addClass(''Model'', smalltalk.Object, [''announcer''], ''Flow'');
smalltalk.addMethod(
unescape(''_announcer''),
smalltalk.method({
selector: unescape(''announcer''),
fn: function () {
var self=this;
return (($receiver = self[''@announcer'']) == nil || $receiver == undefined) ? (function(){return smalltalk.send(self, "_initializeAnnouncer", []);})() : $receiver;
return self;}
}),
smalltalk.Model);

smalltalk.addMethod(
unescape(''_initializeAnnouncer''),
smalltalk.method({
selector: unescape(''initializeAnnouncer''),
fn: function () {
var self=this;
return (self[''@announcer'']=smalltalk.send((smalltalk.Announcer || Announcer), "_new", []));
return self;}
}),
smalltalk.Model);

smalltalk.addMethod(
unescape(''_announce_''),
smalltalk.method({
selector: unescape(''announce%3A''),
fn: function (anAnnouncement) {
var self=this;
smalltalk.send(smalltalk.send(self, "_announcer", []), "_announce_", [anAnnouncement]);
return self;}
}),
smalltalk.Model);

smalltalk.addMethod(
unescape(''_on_do_''),
smalltalk.method({
selector: unescape(''on%3Ado%3A''),
fn: function (anAnnouncementClass, aReactionBlock) {
var self=this;
return smalltalk.send(smalltalk.send(self, "_announcer", []), "_on_do_", [anAnnouncementClass, aReactionBlock]);
return self;}
}),
smalltalk.Model);


smalltalk.addMethod(
unescape(''_initialize''),
smalltalk.method({
selector: unescape(''initialize''),
fn: function () {
var self=this;
smalltalk.send((smalltalk.Package || Package), "_defaultCommitPathSt_", [unescape("/amberMP/commitCode")]);
smalltalk.send((smalltalk.Package || Package), "_defaultCommitPathJs_", [unescape("/amberMP/commitCode")]);
return self;}
}),
smalltalk.Model.klass);

smalltalk.addMethod(
unescape(''_basePath''),
smalltalk.method({
selector: unescape(''basePath''),
fn: function () {
var self=this;
return "api";
return self;}
}),
smalltalk.Model.klass);

smalltalk.addMethod(
unescape(''_path''),
smalltalk.method({
selector: unescape(''path''),
fn: function () {
var self=this;
return smalltalk.send(smalltalk.send(smalltalk.send(self, "_basePath", []), "__comma", [unescape("/")]), "__comma", [smalltalk.send(self, "_modelPath", [])]);
return self;}
}),
smalltalk.Model.klass);

smalltalk.addMethod(
unescape(''_modelPath''),
smalltalk.method({
selector: unescape(''modelPath''),
fn: function () {
var self=this;
return smalltalk.send(self, "_name", []);
return self;}
}),
smalltalk.Model.klass);

smalltalk.addMethod(
unescape(''_jsonIgnoreInstanceVariables''),
smalltalk.method({
selector: unescape(''jsonIgnoreInstanceVariables''),
fn: function (){
var self=this;
return smalltalk.send(smalltalk.send(self, "_jsonIgnoreInstanceVariables", [], smalltalk.Object.klass), "__comma", [["announcer"]]);
return self;}
}),
smalltalk.Model.klass);


smalltalk.addClass(''Presenter'', smalltalk.Widget, [''children'', ''model'', ''announcer'', ''parent'', ''visible'', ''wrapper''], ''Flow'');
smalltalk.addMethod(
unescape(''_announcer''),
smalltalk.method({
selector: unescape(''announcer''),
fn: function () {
var self=this;
return (($receiver = self[''@announcer'']) == nil || $receiver == undefined) ? (function(){return smalltalk.send(self, "_initializeAnnouncer", []);})() : $receiver;
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_initializeAnnouncer''),
smalltalk.method({
selector: unescape(''initializeAnnouncer''),
fn: function () {
var self=this;
return (self[''@announcer'']=smalltalk.send((smalltalk.Announcer || Announcer), "_new", []));
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_announce_''),
smalltalk.method({
selector: unescape(''announce%3A''),
fn: function (anAnnouncement) {
var self=this;
smalltalk.send(smalltalk.send(self, "_announcer", []), "_announce_", [anAnnouncement]);
((($receiver = smalltalk.send(anAnnouncement, "_isBubbling", [])).klass === smalltalk.Boolean) ? ($receiver ? (function(){return (($receiver = smalltalk.send(self, "_parent", [])) !!= nil && $receiver !!= undefined) ? (function(){return smalltalk.send(smalltalk.send(self, "_parent", []), "_announce_", [anAnnouncement]);})() : nil;})() : nil) : smalltalk.send($receiver, "_ifTrue_", [(function(){return (($receiver = smalltalk.send(self, "_parent", [])) !!= nil && $receiver !!= undefined) ? (function(){return smalltalk.send(smalltalk.send(self, "_parent", []), "_announce_", [anAnnouncement]);})() : nil;})]));
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_hasParent''),
smalltalk.method({
selector: unescape(''hasParent''),
fn: function () {
var self=this;
return smalltalk.send(self[''@parent''], "_notNil", []);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_parent''),
smalltalk.method({
selector: unescape(''parent''),
fn: function () {
var self=this;
return self[''@parent''];
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_parent_''),
smalltalk.method({
selector: unescape(''parent%3A''),
fn: function (aKdWidget) {
var self=this;
(self[''@parent'']=aKdWidget);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_children''),
smalltalk.method({
selector: unescape(''children''),
fn: function () {
var self=this;
return (($receiver = self[''@children'']) == nil || $receiver == undefined) ? (function(){return smalltalk.send(self, "_initializeChildren", []);})() : $receiver;
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_initializeChildren''),
smalltalk.method({
selector: unescape(''initializeChildren''),
fn: function () {
var self=this;
return (self[''@children'']=smalltalk.send((smalltalk.Dictionary || Dictionary), "_new", []));
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_at_ifAbsent_''),
smalltalk.method({
selector: unescape(''at%3AifAbsent%3A''),
fn: function (aKey, aBlock) {
var self=this;
return smalltalk.send(smalltalk.send(self, "_children", []), "_at_ifAbsent_", [aKey, aBlock]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_at_''),
smalltalk.method({
selector: unescape(''at%3A''),
fn: function (aKey) {
var self=this;
return smalltalk.send(smalltalk.send(self, "_children", []), "_at_", [aKey]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_allSubChildren''),
smalltalk.method({
selector: unescape(''allSubChildren''),
fn: function () {
var self=this;
return smalltalk.send(self, "_addAllSubChildrenTo_", [smalltalk.send((smalltalk.Set || Set), "_new", [])]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_addAllSubChildrenTo_''),
smalltalk.method({
selector: unescape(''addAllSubChildrenTo%3A''),
fn: function (aSet) {
var self=this;
var subChildren=nil;
(subChildren=smalltalk.send(smalltalk.send(self, "_children", []), "_values", []));
smalltalk.send(subChildren, "_do_", [(function(aKdWidget){return (($receiver = aKdWidget) !!= nil && $receiver !!= undefined) ? (function(){smalltalk.send(aSet, "_add_", [aKdWidget]);return smalltalk.send(aKdWidget, "_addAllSubChildrenTo_", [aSet]);})() : nil;})]);
return aSet;
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_at_put_''),
smalltalk.method({
selector: unescape(''at%3Aput%3A''),
fn: function (aKey, aPresenter){
var self=this;
smalltalk.send(self, "_childrenAt_put_", [aKey, aPresenter]);
return aPresenter;
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_childrenAt_''),
smalltalk.method({
selector: unescape(''childrenAt%3A''),
fn: function (aKey) {
var self=this;
return smalltalk.send(self, "_at_", [aKey]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_doesNotUnderstand_''),
smalltalk.method({
selector: unescape(''doesNotUnderstand%3A''),
fn: function (aMessage){
var self=this;
return ((($receiver = smalltalk.send(self, "_hasPresenterAt_", [smalltalk.send(smalltalk.send(aMessage, "_selector", []), "_asSymbol", [])])).klass === smalltalk.Boolean) ? ($receiver ? (function(){return smalltalk.send(self, "_at_", [smalltalk.send(smalltalk.send(aMessage, "_selector", []), "_asSymbol", [])]);})() : (function(){return smalltalk.send(self, "_doesNotUnderstand_", [aMessage], smalltalk.Widget);})()) : smalltalk.send($receiver, "_ifTrue_ifFalse_", [(function(){return smalltalk.send(self, "_at_", [smalltalk.send(smalltalk.send(aMessage, "_selector", []), "_asSymbol", [])]);}), (function(){return smalltalk.send(self, "_doesNotUnderstand_", [aMessage], smalltalk.Widget);})]));
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_model_''),
smalltalk.method({
selector: unescape(''model%3A''),
fn: function (aModel) {
var self=this;
smalltalk.send(self, "_setModel_", [aModel]);
smalltalk.send(self, "_onModelUpdated", []);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_onModelUpdated''),
smalltalk.method({
selector: unescape(''onModelUpdated''),
fn: function () {
var self=this;

return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_model''),
smalltalk.method({
selector: unescape(''model''),
fn: function () {
var self=this;
return self[''@model''];
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_setModel_''),
smalltalk.method({
selector: unescape(''setModel%3A''),
fn: function (aModel) {
var self=this;
smalltalk.send(self, "_onAboutToUpdateModel_", [aModel]);
(self[''@model'']=aModel);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_onAboutToUpdateModel_''),
smalltalk.method({
selector: unescape(''onAboutToUpdateModel%3A''),
fn: function (aModel) {
var self=this;

return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_open''),
smalltalk.method({
selector: unescape(''open''),
fn: function (){
var self=this;
smalltalk.send(self, "_onAboutToOpen", []);
smalltalk.send(self, "_paintOnJQuery_", [smalltalk.send("body", "_asJQuery", [])]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_onAboutToOpen''),
smalltalk.method({
selector: unescape(''onAboutToOpen''),
fn: function () {
var self=this;

return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_ifAbsentAt_put_''),
smalltalk.method({
selector: unescape(''ifAbsentAt%3Aput%3A''),
fn: function (aKey, aBlock){
var self=this;
return ((($receiver = smalltalk.send(self, "_hasPresenterAt_", [aKey])).klass === smalltalk.Boolean) ? (!! $receiver ? (function(){return smalltalk.send(self, "_at_put_", [aKey, smalltalk.send(aBlock, "_value", [])]);})() : nil) : smalltalk.send($receiver, "_ifFalse_", [(function(){return smalltalk.send(self, "_at_put_", [aKey, smalltalk.send(aBlock, "_value", [])]);})]));
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_ifAbsentAt_put_andDo_''),
smalltalk.method({
selector: unescape(''ifAbsentAt%3Aput%3AandDo%3A''),
fn: function (aKey, aBlock, anotherBlock){
var self=this;
((($receiver = smalltalk.send(self, "_hasPresenterAt_", [aKey])).klass === smalltalk.Boolean) ? (!! $receiver ? (function(){smalltalk.send(self, "_at_put_", [aKey, smalltalk.send(aBlock, "_value", [])]);return smalltalk.send(anotherBlock, "_value", []);})() : nil) : smalltalk.send($receiver, "_ifFalse_", [(function(){smalltalk.send(self, "_at_put_", [aKey, smalltalk.send(aBlock, "_value", [])]);return smalltalk.send(anotherBlock, "_value", []);})]));
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_componentsAt_put_''),
smalltalk.method({
selector: unescape(''componentsAt%3Aput%3A''),
fn: function (aKey, aPresenter) {
var self=this;
smalltalk.send(smalltalk.send(self, "_children", []), "_at_put_", [aKey, aPresenter]);
((($receiver = smalltalk.send(aPresenter, "_isPresenter", [])).klass === smalltalk.Boolean) ? ($receiver ? (function(){return smalltalk.send(aPresenter, "_parent_", [self]);})() : nil) : smalltalk.send($receiver, "_ifTrue_", [(function(){return smalltalk.send(aPresenter, "_parent_", [self]);})]));
return aPresenter;
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_childrenAt_put_''),
smalltalk.method({
selector: unescape(''childrenAt%3Aput%3A''),
fn: function (aKey, aPresenter) {
var self=this;
smalltalk.send(smalltalk.send(self, "_children", []), "_at_put_", [aKey, aPresenter]);
((($receiver = smalltalk.send(aPresenter, "_isPresenter", [])).klass === smalltalk.Boolean) ? ($receiver ? (function(){return smalltalk.send(aPresenter, "_parent_", [self]);})() : nil) : smalltalk.send($receiver, "_ifTrue_", [(function(){return smalltalk.send(aPresenter, "_parent_", [self]);})]));
return aPresenter;
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_on_do_''),
smalltalk.method({
selector: unescape(''on%3Ado%3A''),
fn: function (anAnnouncementClass, aReactionBlock) {
var self=this;
return smalltalk.send(smalltalk.send(self, "_announcer", []), "_on_do_", [anAnnouncementClass, aReactionBlock]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_newCanvasOn_''),
smalltalk.method({
selector: unescape(''newCanvasOn%3A''),
fn: function (aJQuery) {
var self=this;
return smalltalk.send((smalltalk.HTMLCanvas || HTMLCanvas), "_onJQuery_", [aJQuery]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_isPresenter''),
smalltalk.method({
selector: unescape(''isPresenter''),
fn: function () {
var self=this;
return true;
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_visible''),
smalltalk.method({
selector: unescape(''visible''),
fn: function (){
var self=this;
return (($receiver = self[''@visible'']) == nil || $receiver == undefined) ? (function(){return smalltalk.send(self, "_initializeVisible", []);})() : $receiver;
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_visible_''),
smalltalk.method({
selector: unescape(''visible%3A''),
fn: function (aBoolean){
var self=this;
(self[''@visible'']=aBoolean);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_initializeVisible''),
smalltalk.method({
selector: unescape(''initializeVisible''),
fn: function (){
var self=this;
return (self[''@visible'']=false);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_renderOn_''),
smalltalk.method({
selector: unescape(''renderOn%3A''),
fn: function (html){
var self=this;
(self[''@painted'']=true);
(self[''@wrapper'']=smalltalk.send(html, "_div", []));
(function($rec){smalltalk.send($rec, "_class_", [smalltalk.send(smalltalk.send(self, "_class", []), "_name", [])]);return smalltalk.send($rec, "_with_", [(function(){return smalltalk.send(self, "_paintOn_", [html]);})]);})(self[''@wrapper'']);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_hide''),
smalltalk.method({
selector: unescape(''hide''),
fn: function (){
var self=this;
smalltalk.send(smalltalk.send(self[''@wrapper''], "_asJQuery", []), "_hide", []);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_show''),
smalltalk.method({
selector: unescape(''show''),
fn: function (){
var self=this;
smalltalk.send(smalltalk.send(self[''@wrapper''], "_asJQuery", []), "_show", []);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_asJQuery''),
smalltalk.method({
selector: unescape(''asJQuery''),
fn: function (){
var self=this;
return smalltalk.send(self[''@wrapper''], "_asJQuery", []);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_append_''),
smalltalk.method({
selector: unescape(''append%3A''),
fn: function (aPresenter){
var self=this;
smalltalk.send(self[''@wrapper''], "_append_", [smalltalk.send(aPresenter, "_asJQuery", [])]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_paintOn_''),
smalltalk.method({
selector: unescape(''paintOn%3A''),
fn: function (html){
var self=this;
self;
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_paintOnJQuery_''),
smalltalk.method({
selector: unescape(''paintOnJQuery%3A''),
fn: function (aJQuery){
var self=this;
smalltalk.send(self, "_appendToJQuery_", [aJQuery]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_hasPresenterAt_''),
smalltalk.method({
selector: unescape(''hasPresenterAt%3A''),
fn: function (aKey){
var self=this;
return smalltalk.send(smalltalk.send(self, "_children", []), "_includesKey_", [aKey]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_paint_''),
smalltalk.method({
selector: unescape(''paint%3A''),
fn: function (aPresenter){
var self=this;
smalltalk.send(aPresenter, "_paintOnJQuery_", [smalltalk.send(self, "_asJQuery", [])]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_hideAll''),
smalltalk.method({
selector: unescape(''hideAll''),
fn: function (){
var self=this;
smalltalk.send(smalltalk.send(smalltalk.send(self, "_children", []), "_keys", []), "_do_", [(function(each){var child=nil;
(child=smalltalk.send(self, "_at_ifAbsent_", [each, (function(){return nil;})]));return (($receiver = child) !!= nil && $receiver !!= undefined) ? (function(){return smalltalk.send(child, "_hide", []);})() : nil;})]);
return self;}
}),
smalltalk.Presenter);

smalltalk.addMethod(
unescape(''_wrapper''),
smalltalk.method({
selector: unescape(''wrapper''),
fn: function (){
var self=this;
return self[''@wrapper''];
return self;}
}),
smalltalk.Presenter);


smalltalk.addMethod(
unescape(''_for_''),
smalltalk.method({
selector: unescape(''for%3A''),
fn: function (aModel) {
var self=this;
return (function($rec){smalltalk.send($rec, "_model_", [aModel]);return smalltalk.send($rec, "_yourself", []);})(smalltalk.send(self, "_new", []));
return self;}
}),
smalltalk.Presenter.klass);

smalltalk.addMethod(
unescape(''_open''),
smalltalk.method({
selector: unescape(''open''),
fn: function () {
var self=this;
return smalltalk.send(smalltalk.send(self, "_for_", [smalltalk.send(self, "_defaultModel", [])]), "_open", []);
return self;}
}),
smalltalk.Presenter.klass);

smalltalk.addMethod(
unescape(''_defaultModel''),
smalltalk.method({
selector: unescape(''defaultModel''),
fn: function () {
var self=this;
return nil;
return self;}
}),
smalltalk.Presenter.klass);


smalltalk.addClass(''PersistentModel'', smalltalk.Model, [''id''], ''Flow'');
smalltalk.addMethod(
unescape(''_id_''),
smalltalk.method({
selector: unescape(''id%3A''),
fn: function (aString) {
var self=this;
(self[''@id'']=aString);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_id''),
smalltalk.method({
selector: unescape(''id''),
fn: function () {
var self=this;
return self[''@id''];
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_url''),
smalltalk.method({
selector: unescape(''url''),
fn: function () {
var self=this;
return smalltalk.send(smalltalk.send(smalltalk.send(self, "_path", []), "__comma", [unescape("/")]), "__comma", [smalltalk.send(self[''@id''], "_asString", [])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onAfterCreate_''),
smalltalk.method({
selector: unescape(''onAfterCreate%3A''),
fn: function (x) {
var self=this;
smalltalk.send(self, "_syncWith_", [smalltalk.send(smalltalk.send(self, "_class", []), "_reify_", [x])]);
smalltalk.send(self, "_announce_", [smalltalk.send((smalltalk.ModelCreated || ModelCreated), "_for_", [self])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_update''),
smalltalk.method({
selector: unescape(''update''),
fn: function () {
var self=this;
smalltalk.send((typeof jQuery == ''undefined'' ? nil : jQuery), "_ajax_", [smalltalk.HashedCollection._fromPairs_([smalltalk.send(smalltalk.send(smalltalk.send("url", "__minus_gt", [smalltalk.send(self, "_path", [])]), "__comma", [unescape("/")]), "__comma", [smalltalk.send(self[''@id''], "_asString", [])]),smalltalk.send("type", "__minus_gt", ["PUT"]),smalltalk.send("data", "__minus_gt", [smalltalk.send(self, "_asJSONString", [])]),smalltalk.send("success", "__minus_gt", [(function(x){return smalltalk.send(self, "_onAfterUpdate_", [x]);})]),smalltalk.send("fail", "__minus_gt", [(function(x){return smalltalk.send(self, "_onUpdateFail_", [x]);})]),smalltalk.send("error", "__minus_gt", [(function(x){return smalltalk.send(self, "_onUpdateFail_", [x]);})])])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_delete''),
smalltalk.method({
selector: unescape(''delete''),
fn: function () {
var self=this;
smalltalk.send((typeof jQuery == ''undefined'' ? nil : jQuery), "_ajax_", [smalltalk.HashedCollection._fromPairs_([smalltalk.send("url", "__minus_gt", [smalltalk.send(smalltalk.send(smalltalk.send(self, "_path", []), "__comma", [unescape("/")]), "__comma", [smalltalk.send(self[''@id''], "_asString", [])])]),smalltalk.send("type", "__minus_gt", ["DELETE"]),smalltalk.send("success", "__minus_gt", [(function(x){return smalltalk.send(self, "_onAfterDelete_", [x]);})]),smalltalk.send("fail", "__minus_gt", [(function(x){return smalltalk.send(self, "_onDeleteFail_", [x]);})]),smalltalk.send("error", "__minus_gt", [(function(x){return smalltalk.send(self, "_onDeleteFail_", [x]);})])])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onAfterUpdate_''),
smalltalk.method({
selector: unescape(''onAfterUpdate%3A''),
fn: function (x) {
var self=this;
smalltalk.send(self, "_announce_", [smalltalk.send((smalltalk.ModelUpdated || ModelUpdated), "_for_", [self])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onUpdateFail_''),
smalltalk.method({
selector: unescape(''onUpdateFail%3A''),
fn: function (x) {
var self=this;
smalltalk.send((smalltalk.ModelUpdateError || ModelUpdateError), "_signal_", [smalltalk.send("Could not update ", "__comma", [smalltalk.send(smalltalk.send(self, "_class", []), "_name", [])])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onDeleteFail_''),
smalltalk.method({
selector: unescape(''onDeleteFail%3A''),
fn: function (x){
var self=this;
smalltalk.send((smalltalk.ModelUpdateError || ModelUpdateError), "_signal_", [smalltalk.send("Could not delete ", "__comma", [smalltalk.send(smalltalk.send(self, "_class", []), "_name", [])])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onAfterDelete_''),
smalltalk.method({
selector: unescape(''onAfterDelete%3A''),
fn: function (x) {
var self=this;
smalltalk.send(self, "_announce_", [smalltalk.send((smalltalk.ModelDeleted || ModelDeleted), "_for_", [self])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_path''),
smalltalk.method({
selector: unescape(''path''),
fn: function () {
var self=this;
return smalltalk.send(smalltalk.send(self, "_class", []), "_path", []);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_syncWith_''),
smalltalk.method({
selector: unescape(''syncWith%3A''),
fn: function (aReified){
var self=this;
var toSynchronize=nil;
(toSynchronize=smalltalk.send(smalltalk.send(smalltalk.send(self, "_class", []), "_allInstanceVariableNames", []), "_reject_", [(function(e){return smalltalk.send(smalltalk.send(smalltalk.send(self, "_class", []), "_jsonIgnoreInstanceVariables", []), "_includes_", [e]);})]));
smalltalk.send(toSynchronize, "_do_", [(function(instVarName){return smalltalk.send(self, "_instVarAt_put_", [instVarName, smalltalk.send(aReified, "_at_", [instVarName])]);})]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_refresh''),
smalltalk.method({
selector: unescape(''refresh''),
fn: function () {
var self=this;
smalltalk.send((typeof jQuery == ''undefined'' ? nil : jQuery), "_ajax_", [smalltalk.HashedCollection._fromPairs_([smalltalk.send("url", "__minus_gt", [smalltalk.send(smalltalk.send(smalltalk.send(self, "_path", []), "__comma", [unescape("/")]), "__comma", [smalltalk.send(self[''@id''], "_asString", [])])]),smalltalk.send("type", "__minus_gt", ["GET"]),smalltalk.send("success", "__minus_gt", [(function(x){return smalltalk.send(self, "_onAfterRefresh_", [x]);})]),smalltalk.send("fail", "__minus_gt", [(function(x){return smalltalk.send(self, "_onRefeshFail_", [x]);})]),smalltalk.send("error", "__minus_gt", [(function(x){return smalltalk.send(self, "_onRefreshFail_", [x]);})])])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onRefreshFail_''),
smalltalk.method({
selector: unescape(''onRefreshFail%3A''),
fn: function (x) {
var self=this;
smalltalk.send((smalltalk.ModelRefreshError || ModelRefreshError), "_signal_", [smalltalk.send("Could not refresh ", "__comma", [smalltalk.send(smalltalk.send(self, "_class", []), "_name", [])])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_onAfterRefresh_''),
smalltalk.method({
selector: unescape(''onAfterRefresh%3A''),
fn: function (x) {
var self=this;
smalltalk.send(self, "_syncWith_", [smalltalk.send(smalltalk.send(self, "_class", []), "_reify_", [x])]);
smalltalk.send(self, "_announce_", [smalltalk.send((smalltalk.ModelRefreshed || ModelRefreshed), "_for_", [self])]);
return self;}
}),
smalltalk.PersistentModel);

smalltalk.addMethod(
unescape(''_save''),
smalltalk.method({
selector: unescape(''save''),
fn: function () {
var self=this;
return smalltalk.send(self, "_update", []);
return self;}
}),
smalltalk.PersistentModel);


smalltalk.addMethod(
unescape(''_atId_''),
smalltalk.method({
selector: unescape(''atId%3A''),
fn: function (anId) {
var self=this;
return smalltalk.send(self, "_read_", [anId]);
return self;}
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_read_''),
smalltalk.method({
selector: unescape(''read%3A''),
fn: function (anId) {
var self=this;
smalltalk.send((typeof jQuery == ''undefined'' ? nil : jQuery), "_ajax_", [smalltalk.HashedCollection._fromPairs_([smalltalk.send("url", "__minus_gt", [smalltalk.send(smalltalk.send(smalltalk.send(self, "_path", []), "__comma", [unescape("/")]), "__comma", [anId])]),smalltalk.send("type", "__minus_gt", ["GET"]),smalltalk.send("sucess", "__minus_gt", [(function(x){return smalltalk.send(self, "_onAfterRead_", [x]);})]),smalltalk.send("fail", "__minus_gt", [(function(x){return smalltalk.send(self, "_onReadFail_", [x]);})]),smalltalk.send("error", "__minus_gt", [(function(x){return smalltalk.send(self, "_onReadFail_", [x]);})])])]);
return self;}
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_create''),
smalltalk.method({
selector: unescape(''create''),
fn: function (){
var self=this;
var newInstance=nil;
(newInstance=smalltalk.send(smalltalk.send(self, "_basicNew", []), "_initialize", []));
smalltalk.send((typeof jQuery == ''undefined'' ? nil : jQuery), "_ajax_", [smalltalk.HashedCollection._fromPairs_([smalltalk.send("url", "__minus_gt", [smalltalk.send(self, "_path", [])]),smalltalk.send("type", "__minus_gt", ["POST"]),smalltalk.send("success", "__minus_gt", [(function(x){return smalltalk.send(newInstance, "_onAfterCreate_", [x]);})]),smalltalk.send("fail", "__minus_gt", [(function(x){return smalltalk.send(self, "_onCreateFail_", [x]);})]),smalltalk.send("error", "__minus_gt", [(function(x){return smalltalk.send(self, "_onCreateFail_", [x]);})])])]);
return newInstance;
return self;}
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_onCreateFail_''),
smalltalk.method({
selector: unescape(''onCreateFail%3A''),
fn: function (x) {
var self=this;
smalltalk.send((smalltalk.ModelCreateError || ModelCreateError), "_signal_", [smalltalk.send(smalltalk.send(smalltalk.send("Could not create ", "__comma", [smalltalk.send(self, "_name", [])]), "__comma", [":  "]), "__comma", [smalltalk.send(x, "_responseText", [])])]);
return self;}
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_onReadFail_''),
smalltalk.method({
selector: unescape(''onReadFail%3A''),
fn: function (x) {
var self=this;
smalltalk.send((smalltalk.ModelReadError || ModelReadError), "_signal_", [smalltalk.send(smalltalk.send(smalltalk.send("Could not read ", "__comma", [smalltalk.send(self, "_name", [])]), "__comma", [":  "]), "__comma", [smalltalk.send(x, "_responseText", [])])]);
return self;}
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_onAfterRead_''),
smalltalk.method({
selector: unescape(''onAfterRead%3A''),
fn: function (x) {
var self=this;
return smalltalk.send(self, "_reify_", [x]);
return self;}
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_reify_''),
smalltalk.method({
selector: unescape(''reify%3A''),
fn: function (aJSONString) {
var self=this;
return smalltalk.send((smalltalk.JSON || JSON), "_parse_", [aJSONString]);
return self;}
}),
smalltalk.PersistentModel.klass);

smalltalk.addMethod(
unescape(''_new''),
smalltalk.method({
selector: unescape(''new''),
fn: function () {
var self=this;
return smalltalk.send(self, "_create", []);
return self;}
}),
smalltalk.PersistentModel.klass);


'! !

Object subclass: #Flow
	instanceVariableNames: ''
	classVariableNames: 'Packages Repository Synchronizable'
	poolDictionaries: ''
	category: 'Flow-Core'!

!Flow commentStamp: 'sebsatiansastre 3/7/2012 17:38' prior: 0!
AmberMP this software adds to Amber a Model-Presenter pattern.
This class is only a development support so it can be used with Kaliningrad.

!

!Flow class methodsFor: 'actions' stamp: 'sebsatiansastre 3/8/2012 11:02'!
addPackage: aPackageName

	(self packages includes: aPackageName) ifFalse:[
		self packages add: aPackageName]! !

!Flow class methodsFor: 'actions' stamp: 'sebsatiansastre 3/8/2012 11:10'!
addSynchronizable: anObject

	(self synchronizable includes: anObject) ifFalse:[
		self synchronizable add: anObject]! !

!Flow class methodsFor: 'accessing' stamp: 'sebsatiansastre 3/8/2012 11:17'!
defaultRepository

	^ MCDirectoryRepository new
		directory: (FileDirectory default / 'src');
		yourself
! !

!Flow class methodsFor: 'initialization' stamp: 'sebsatiansastre 3/8/2012 11:01'!
initializePackages

	^ Packages := OrderedCollection new! !

!Flow class methodsFor: 'initialization' stamp: 'sebsatiansastre 3/8/2012 11:14'!
initializeRepository

	^ Repository := self defaultRepository! !

!Flow class methodsFor: 'initialization' stamp: 'sebsatiansastre 3/8/2012 11:08'!
initializeSynchronizable

	^ Synchronizable := OrderedCollection new! !

!Flow class methodsFor: 'accessing' stamp: 'sebsatiansastre 3/8/2012 11:01'!
packages

	^ Packages ifNil:[self initializePackages]! !

!Flow class methodsFor: 'accessing' stamp: 'sebsatiansastre 3/8/2012 11:04'!
repository 

	^ Repository ifNil:[self initializeRepository]! !

!Flow class methodsFor: 'accessing' stamp: 'sebsatiansastre 3/8/2012 11:09'!
repository: aRepository 

	Repository := aRepository! !

!Flow class methodsFor: 'accessing' stamp: 'sebsatiansastre 3/8/2012 11:19'!
repositoryOnPath: aString

	^ MCDirectoryRepository new
		directory: (FileDirectory on: aString);
		yourself
! !

!Flow class methodsFor: 'actions' stamp: 'sebsatiansastre 3/8/2012 11:08'!
reset

	Packages := nil.
	Repository := nil.
	Synchronizable := nil! !

!Flow class methodsFor: 'actions' stamp: 'sebastiansastre 3/10/2012 21:04'!
restart

	self stop.
	WAKomEncoded39 startOn: 8080
! !

!Flow class methodsFor: 'actions' stamp: 'sebsatiansastre 3/8/2012 11:18'!
save: aPackageName
	
	| repo workingCopy |

	repo := self repository.
	repo directory assureExistence. 
	workingCopy := (MCPackage named: aPackageName) workingCopy.
	workingCopy newVersion ifNotNil: [:v |
		Cursor wait showWhile: [
			[ repo storeVersion: v ]
				on: Error
				do: [ :err | 	
					(MCVersionInspector new
						label: ('An error occured while saving ', v info name);
						version: v) show.
					^ self inform: err messageText ] ].
		(MCVersionInspector new version: v) show.
		MCCacheRepository default cacheAllFileNamesDuring: 
			[repo cacheAllFileNamesDuring: 
				[v allAvailableDependenciesDo:
					[:dep |
					(repo includesVersionNamed: dep info name)
						ifFalse: [repo storeVersion: dep]]]]]! !

!Flow class methodsFor: 'actions' stamp: 'sebsatiansastre 3/8/2012 11:06'!
saveAll

	self sync.
	
	self packages do:[:packageName|
		self save: packageName]! !

!Flow class methodsFor: 'actions' stamp: 'sebsatiansastre 3/8/2012 11:20'!
setPath: aString

	Repository := self repositoryOnPath: aString! !

!Flow class methodsFor: 'actions' stamp: 'sebastiansastre 3/10/2012 21:04'!
start

	self restart! !

!Flow class methodsFor: 'actions' stamp: 'sebastiansastre 3/10/2012 21:04'!
stop

	WAKomEncoded39 stop! !

!Flow class methodsFor: 'actions' stamp: 'sebsatiansastre 3/8/2012 11:10'!
sync

	self synchronizable do:[:anObject|
		anObject sync]! !

!Flow class methodsFor: 'accessing' stamp: 'sebsatiansastre 3/8/2012 11:07'!
synchronizable 

	^ Synchronizable ifNil:[self initializeSynchronizable]! !

Object subclass: #FlowPersistentModel
	instanceVariableNames: 'data'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Models'!

!FlowPersistentModel commentStamp: 'sebastiansastre 3/15/2012 21:49' prior: 0!
Flow models are meant to be treated as aggregates.

We don't care here about identity.

Why?

Because the idea is that you keep these objects small and uncomplicated enough to be more efficienty serialized/deserialized in JSON for using it is some sort of noSQL persistence support.!

!FlowPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 4/21/2012 11:46'!
atId: anId

	^ ((self database objectsAt: self ifAbsent:[^nil])
		detect: [:e| e id = anId] 
		ifNone: [^nil]) onRead
	! !

!FlowPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 3/11/2012 22:49'!
database
	"Answers the database used by this kind of models.
	Subclasses should override if appropiate"
	^ FlowSimpleDb instance! !

!FlowPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 11:30'!
findAll 

	^ self database objectsAt: self! !

!FlowPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 23:39'!
fromJson: someJson

	^ self basicNew data: (Json readFrom: someJson readStream)! !

!FlowPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 10:52'!
fromJsonObject: aJsonObject

	^ self basicNew 
		initializeOn: aJsonObject;
		yourself 
! !

!FlowPersistentModel class methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 10:00'!
fromReified: aJsonObject 

	^ self fromJsonObject: aJsonObject ! !

!FlowPersistentModel class methodsFor: 'actions' stamp: 'sebastiansastre 3/4/2012 19:31'!
makeId

	^ UUID new asString36! !

!FlowPersistentModel methodsFor: 'testing' stamp: 'sebastiansastre 3/17/2012 14:43'!
= aModel 
	"Answers true if this model is the same as aModel.
	Note: as you can see, this won't care about identity (yes, that's intentional)."
	^ self id = aModel id! !

!FlowPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 11:58'!
asJsonString
	
	self onAboutToJSON.
	
	^ super asJsonString! !

!FlowPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 10:43'!
at: aKey

	^ data at: aKey! !

!FlowPersistentModel methodsFor: 'actions' stamp: 'sebastiansastre 3/24/2012 10:43'!
at: aKey put: anObject

	^ data at: aKey put: anObject! !

!FlowPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 11:43'!
createdOn 

	| selector expects object|

	selector := #createdOn.
	expects := DateAndTime.

	object := self at: selector asString.
	object ifNil:[^nil].

	(object isKindOf: expects) ifTrue:[^object].

	^ self at: selector asString put: (self dateAndTimeAt: selector).
! !

!FlowPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 23:32'!
data

	^ data! !

!FlowPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/18/2012 23:32'!
data: aJsonObject

	data := aJsonObject 
	! !

!FlowPersistentModel methodsFor: 'actions' stamp: 'sebastiansastre 3/24/2012 11:47'!
dateAndTimeAt: aSelector

	^ DateAndTime fromString: (self at: aSelector)! !

!FlowPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/19/2012 21:56'!
doesNotUnderstand: aMessage
	"The idea behind this DNU is to use the selector as setters or getter 
	delegating to data (aJsonObject)"
	
	| key |
	key := aMessage selector.
	key isUnary ifTrue: [^ data at: key asString].
	^ (key isKeyword and: [(key occurrencesOf: $:) = 1])
		ifTrue: [key := key allButLast asSymbol.
				data at: key asString put: aMessage arguments first]
		ifFalse: [super doesNotUnderstand: aMessage]
				
	! !

!FlowPersistentModel methodsFor: 'actions' stamp: 'sebastiansastre 3/27/2012 10:54'!
fresh

	^ self class atId: self id ! !

!FlowPersistentModel methodsFor: 'initialization' stamp: 'sebastiansastre 3/19/2012 21:59'!
initialize

	super initialize.
	
	data := JsonObject new.
	
	self id: self class makeId.
	
	"We include the class name so we have the minimal metadata at consuming time"
	self class: self class name.
	self createdOn: DateAndTime now.
! !

!FlowPersistentModel methodsFor: 'initialization' stamp: 'sebastiansastre 3/24/2012 10:51'!
initializeOn: aJsonObject 
	"Makes this model to be what aJsonObject dictates it to be.
	Unwraps data if necessary."
	
	(aJsonObject at: '@data') isNil 
		ifTrue:[data := aJsonObject]
		ifFalse:[data := aJsonObject at: '@data']
	 ! !

!FlowPersistentModel methodsFor: 'testing' stamp: 'sebastiansastre 3/27/2012 10:43'!
isPersistentModel

	^ true! !

!FlowPersistentModel methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 23:53'!
jsonWriteOn: aStream

	data jsonWriteOn: aStream! !

!FlowPersistentModel methodsFor: 'reactions' stamp: 'sebastiansastre 3/27/2012 19:45'!
onAboutToJSON
	"This model is about to be stringified in JSON.
	All inst var values that are expected to be model objects, need to be instanciated as expected."

	data keys do:[:key| |value|
		value := data at: key.
		"If the value is a js object proxy, we assume it's the data for a (composed) flow model, 
		so we call the getter in this receiver expecting to get it properly instantiated (because that's our convention here)."
		(value isKindOf: JsonObject) ifTrue:[
			(self respondsTo: key asSymbol) ifFalse:[FlowModelGetterIsMissing signal: 'Cannot find getter for: ', key].
			value := self perform: key asSymbol.
			value onAboutToJSON.
			self at: key put: value]].
	! !

!FlowPersistentModel methodsFor: 'reactions' stamp: 'sebastiansastre 3/27/2012 10:48'!
onRead
	"This model has been readed.
	All inst var values that are expected to be model objects, need to be re-instanciated."

	data keys do:[:key| |value|
		value := data at: key.
		value isPersistentModel ifTrue:[
			value := value fresh.
			self at: key put: value]].
	! !

!FlowPersistentModel methodsFor: 'accessing' stamp: 'sebastiansastre 3/24/2012 10:41'!
subModelAt: aSelector
	"Answers the reified submodel (instantiating if necessary)."
	
	| subModelData class |

	subModelData := data at: aSelector.
	subModelData ifNil:[^nil].

	subModelData := subModelData at: '@data'.
	subModelData ifNil:[^ FlowModelDataError signal: 'Cannot find ', aSelector,'''s data'].
	
	class := subModelData at: 'class'.
	class := Smalltalk at: class asSymbol ifAbsent:[
		FlowModelMetadataError signal: 'Cannot find ',aSelector,'''s metadata'].
	
	^ class fromReified: subModelData	! !

FlowPersistentModel subclass: #ToDo
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Models'!

FlowPersistentModel subclass: #ToDoList
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Models'!

Object subclass: #FlowSimpleDb
	instanceVariableNames: 'objects'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Flow-Core'!

FlowSimpleDb class
	instanceVariableNames: 'instance'!

!FlowSimpleDb commentStamp: 'sebastiansastre 3/15/2012 21:25' prior: 0!
This is a simple singleton holding the PersistentModel's you send to them.

Please note that, by now, we only care about persisten models at a level of the "valuesof things"

Don't expect identity to be the rule to compare things here but equal values.

Example: 

	guy := PersonMock new id: '1234'; yourself.
	dude := PersonMock new id: '1234'; yourself.
	
	guy == dude "this is false"
	guy = dude "this is true"
	
	db add: dude.   
	db exists: dude.  "this is true"
	db exists: guy.  "this is also true"

	db remove: guy.
	db exists: dude.  "this is false now"
	db exists: guy.  "this is false now"
			
!

FlowSimpleDb class
	instanceVariableNames: 'instance'!

!FlowSimpleDb class methodsFor: 'initialization' stamp: 'sebastiansastre 3/4/2012 19:15'!
initializeInstance

	^ instance := self new! !

!FlowSimpleDb class methodsFor: 'accessing' stamp: 'sebsatiansastre 3/7/2012 15:09'!
instance

	^ instance ifNil:[self initializeInstance]! !

!FlowSimpleDb class methodsFor: 'actions' stamp: 'sebastiansastre 3/4/2012 19:15'!
reset

	instance := nil! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/17/2012 16:25'!
add: aPersistentModel
	
	| collection |
	
	collection := self objects 
					at: aPersistentModel class 
					ifAbsentPut:[OrderedCollection new].

	(collection includes: aPersistentModel) ifFalse:[
		self addSubModelsFor: aPersistentModel.
		collection add: aPersistentModel].
	
	^ aPersistentModel! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/18/2012 23:37'!
addSubModelsFor: aPersistentModel
	"If any of the values in aPersistentModel is also aFlowPersistentModel,
	then add it to the database."

	aPersistentModel data values do:[:each|
		(each isKindOf: FlowPersistentModel) ifTrue:[
			self add: each]]
	
	! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 3/17/2012 19:28'!
at: anId of: aPersistentModelClass

	^ (self objectsAt: aPersistentModelClass ifAbsent:[^nil])
		detect: [:e| e id = anId] 
		ifNone: [nil]! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:43'!
create: anObject
	
	^ self save: anObject! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/23/2012 20:34'!
delete: aPersistentModel

	^ self remove: aPersistentModel! !

!FlowSimpleDb methodsFor: 'testing' stamp: 'sebastiansastre 3/15/2012 21:19'!
exists: aPersistentModel

	| instances |
	instances := self objects 
					at: aPersistentModel class 
					ifAbsent:[nil].

	^ instances isNil 
		ifTrue: [false] 
		ifFalse: [instances includes: aPersistentModel]! !

!FlowSimpleDb methodsFor: 'initialization' stamp: 'sebastiansastre 3/9/2012 18:15'!
initializeObjects

	^ objects := Dictionary new! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 3/9/2012 18:21'!
objects 

	^ objects ifNil:[self initializeObjects]! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 3/9/2012 18:17'!
objectsAt: aClass

	^ self objects at: aClass ifAbsent:[nil]! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 3/17/2012 19:27'!
objectsAt: aClass ifAbsent: aBlock

	^ self objects at: aClass ifAbsent: aBlock! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:43'!
readAt: anId of: aPersistentModelClass
 
	^ self at: anId of: aPersistentModelClass ! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:19'!
remove: aPersistentModel
	
	| collection |
	
	collection := self objects 
					at: aPersistentModel class 
					ifAbsentPut:[OrderedCollection new].

	^ collection remove: aPersistentModel ifAbsent:[nil]! !

!FlowSimpleDb methodsFor: 'accessing' stamp: 'sebastiansastre 3/15/2012 21:57'!
replace: aPersistentModel
	"Replaces the stored instance that has 
	the same id than aPersistentModel with aPersistentModel."

	| instances old |

	instances := self objectsAt: aPersistentModel class.
	old := instances detect: [:e| e id = aPersistentModel id].

	instances replaceAll: old with: aPersistentModel.
	
	^ aPersistentModel 
	! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:19'!
save: anObject
	
	^ self update: anObject! !

!FlowSimpleDb methodsFor: 'actions' stamp: 'sebastiansastre 3/15/2012 21:47'!
update: aPersistentModel

	^ (self exists: aPersistentModel)
		ifTrue: [self replace: aPersistentModel]
		ifFalse: [self add: aPersistentModel]! !

!Object methodsFor: '*flow' stamp: 'sebastiansastre 3/27/2012 10:44'!
isPersistentModel

	^ false! !

!KOAmberSTFileLibrary methodsFor: '*Flow-Announcements' stamp: 'sebastiansastre 3/13/2012 23:34'!
FlowAnnouncementsSt
	^ 'Smalltalk current createPackage: ''Flow-Announcements'' properties: #{}!!
Object subclass: #Announcement
	instanceVariableNames: ''subject isBubbling''
	category: ''Flow-Announcements''!!

!!Announcement methodsFor: ''accessing''!!

subject

	^ subject
!!

subject: anObject

	subject := anObject
!! !!

!!Announcement methodsFor: ''actions''!!

stop
	
	isBubbling := false
!! !!

!!Announcement methodsFor: ''initialization''!!

initialize

	super initialize.
	
	isBubbling := true
!! !!

!!Announcement methodsFor: ''testing''!!

isBubbling
	
	^ isBubbling
!! !!

!!Announcement class methodsFor: ''not yet classified''!!

for: anObject

	^ self new
		subject: anObject;
		yourself
!! !!

Announcement subclass: #AjaxSuccess
	instanceVariableNames: ''''
	category: ''Flow-Announcements''!!

Announcement subclass: #ModelUpdated
	instanceVariableNames: ''''
	category: ''Flow-Announcements''!!

Announcement subclass: #ModelCreated
	instanceVariableNames: ''''
	category: ''Flow-Announcements''!!

Announcement subclass: #ModelDeleted
	instanceVariableNames: ''''
	category: ''Flow-Announcements''!!

Announcement subclass: #ModelRefreshed
	instanceVariableNames: ''''
	category: ''Flow-Announcements''!!

'! !

!KOAmberSTFileLibrary methodsFor: '*Flow-Errors' stamp: 'sebastiansastre 3/13/2012 23:34'!
FlowErrorsSt
	^ 'Smalltalk current createPackage: ''Flow-Errors'' properties: #{}!!
Error subclass: #FlowError
	instanceVariableNames: ''''
	category: ''Flow-Errors''!!

FlowError subclass: #ModelUpdateError
	instanceVariableNames: ''''
	category: ''Flow-Errors''!!

FlowError subclass: #ModelReadError
	instanceVariableNames: ''''
	category: ''Flow-Errors''!!

FlowError subclass: #ModelDeleteError
	instanceVariableNames: ''''
	category: ''Flow-Errors''!!

FlowError subclass: #ModelCreateError
	instanceVariableNames: ''''
	category: ''Flow-Errors''!!

FlowError subclass: #AjaxError
	instanceVariableNames: ''''
	category: ''Flow-Errors''!!

FlowError subclass: #ModelRefeshError
	instanceVariableNames: ''''
	category: ''Flow-Errors''!!

'! !

!KOAmberSTFileLibrary methodsFor: '*Flow' stamp: 'sebastiansastre 3/15/2012 14:38'!
FlowSt
	^ 'Smalltalk current createPackage: ''Flow'' properties: #{}!!
Object subclass: #Model
	instanceVariableNames: ''announcer''
	category: ''Flow''!!
!!Model commentStamp!!
Models are meant to be used as local-only (at the DOM) objects that should be transident in nature (not-persistent)!!

!!Model methodsFor: ''accessing''!!

announcer

	^ announcer ifNil:[self initializeAnnouncer]
!! !!

!!Model methodsFor: ''actions''!!

announce: anAnnouncement

	self announcer announce: anAnnouncement
!!

on: anAnnouncementClass do: aReactionBlock

	^ self announcer on: anAnnouncementClass do: aReactionBlock
!! !!

!!Model methodsFor: ''initialization''!!

initializeAnnouncer

	^ announcer := Announcer new
!! !!

!!Model class methodsFor: ''accessing''!!

basePath
	"Answers the base path for the application''s API.
	Subclasses should override if appropriate."

	"By default we use the AmberMP''s default"
	^ ''api''
!!

path
	^ self basePath,''/'', self modelPath
!!

modelPath
	"Answers the path for posting instances of this model.
	Subclasses should override if appropriate."

	^ self name
!!

jsonIgnoreInstanceVariables
	"Answers the names of the instance variables that have to be ignored when serializing/reifying this objects to/from json.
	Subclasses should override when appropiate (but don''t forget concatenation with super''s)."

	^ super jsonIgnoreInstanceVariables, 
	#(''announcer'')
!! !!

!!Model class methodsFor: ''initialization''!!

initialize

	"Set amber to commit using the AmberMP''s API"
	Package defaultCommitPathSt: ''/amberMP/commitCode''.
	Package defaultCommitPathJs: ''/amberMP/commitCode''
!! !!

Widget subclass: #Presenter
	instanceVariableNames: ''children model announcer parent visible wrapper''
	category: ''Flow''!!

!!Presenter methodsFor: ''accessing''!!

announcer

	^ announcer ifNil:[self initializeAnnouncer]
!!

announce: anAnnouncement
	"Announce (bubbling through parents)"
	
	self announcer announce: anAnnouncement.

	anAnnouncement isBubbling ifTrue:[
		self parent ifNotNil:[
			self parent announce: anAnnouncement]]
!!

parent

	^ parent
!!

parent: aKdWidget

	parent := aKdWidget
!!

children

	^ children ifNil:[self initializeChildren]
!!

at: aKey ifAbsent: aBlock

	^ self children at: aKey ifAbsent: aBlock
!!

at: aKey
	"Answers the (sub)widget stored in this receiver at aKey"
	^ self children at: aKey
!!

allSubChildren

	^ self addAllSubChildrenTo: Set new
!!

addAllSubChildrenTo: aSet
	"Adds all the sub children of the receiver to aSet."
	
	| subChildren |
	
	subChildren := self children values.
	subChildren do:[:aKdWidget|
		aKdWidget ifNotNil:[
			aSet add: aKdWidget.
			aKdWidget addAllSubChildrenTo: aSet]].
	
	^ aSet
!!

at: aKey put: aPresenter 

	self childrenAt: aKey put: aPresenter.

	^ aPresenter
!!

childrenAt: aKey
	"Answers the (sub)widget that the receiver has stored at aKey"
	^ self at: aKey
!!

model
	
	^ model
!!

ifAbsentAt: aKey put: aBlock
	"Conditionally (to its abscence) adds the presenter returned by aBlock
	as (sub)widget of this widget"
	
	^ (self hasPresenterAt: aKey) ifFalse:[
		self at: aKey put: aBlock value]
!!

ifAbsentAt: aKey put: aBlock andDo: anotherBlock
	"Conditionally (to its abscence) adds the presenter answered by aBlock
	as (sub)widget of this widget and then evaluates anotherBlock.
	Note: the typical applicability of anotherBlock is the configuration 
	of the observation of sub widgets'' announcements"
	
	(self hasPresenterAt: aKey) ifFalse:[
		self at: aKey put: aBlock value.
		anotherBlock value]
!!

componentsAt: aKey put: aPresenter 
	"Answers the presenter recently added to the receiver
	and with this receiver as its parent."
	
	self children at: aKey put: aPresenter.
	aPresenter isPresenter ifTrue: [aPresenter parent: self].
	^ aPresenter
!!

childrenAt: aKey put: aPresenter 
	"Answers the presenter recently added to the receiver
	(with this receiver as its parent)."
	
	self children at: aKey put: aPresenter.
	aPresenter isPresenter ifTrue: [aPresenter parent: self].
	^ aPresenter
!!

on: anAnnouncementClass do: aReactionBlock

	^ self announcer on: anAnnouncementClass do: aReactionBlock
!!

visible
	"Answers true if this presenter should be visible 
	as soon as it get''s rendered (or not, in which case,
	 it will be rendered while staying hidden)."
	^ visible ifNil:[self initializeVisible]
!!

visible: aBoolean
	"Sets if this presenter should be visible 
	as soon as it get''s rendered (or not, in which case,
	 it will be rendered while staying hidden)."
	visible := aBoolean
!!

asJQuery
	
	^ wrapper asJQuery
!!

wrapper

	^ wrapper
!! !!

!!Presenter methodsFor: ''actions''!!

doesNotUnderstand: aMessage

	^ (self hasPresenterAt: aMessage selector asSymbol)
		ifTrue:[self at: aMessage selector asSymbol]
		ifFalse:[super doesNotUnderstand: aMessage]
!!

setModel: aModel
	"This widget is going updating its model to aModel"

	self onAboutToUpdateModel: aModel.

	model := aModel
!!

open
	
	self onAboutToOpen.

	self paintOnJQuery: ''body'' asJQuery
!!

newCanvasOn: aJQuery
	"Answers a new canvas based on the given aJQuery element."

	^ HTMLCanvas onJQuery: aJQuery
!!

hide

	wrapper asJQuery hide
!!

show

	wrapper asJQuery show
!!

append: aPresenter
	"Appends aPresenter to this presenter.
	Note: it''s actually appended to the wrapper of this guy."

	wrapper append: aPresenter asJQuery
!!

paint: aPresenter
	"Paints the (sub)preenter aPresenter in this (parent)presenter."

	aPresenter paintOnJQuery: self asJQuery
!!

hideAll
	"Hide all the children.
	Note: it''s done in such a way that if they don''t exist they won''t evoked."

	self children keys do:[:each| |child|
		child := self at: each ifAbsent:[nil].
		child ifNotNil:[
			child hide]].
!! !!

!!Presenter methodsFor: ''initialization''!!

initializeAnnouncer

	^ announcer := Announcer new
!!

initializeChildren

	^ children := Dictionary new
!!

model: aModel
	"Set the model of this widget"

	self setModel: aModel.
	
	self onModelUpdated
!!

initializeVisible
	"We set the visible default in false so the application''s
	behavior can decide elsewere how (and if) this presenter should appear."

	^ visible := false
!! !!

!!Presenter methodsFor: ''reactions''!!

onModelUpdated
	"The model has been updated.
	This is your chance to react about it."
	
	"no-op"
!!

onAboutToUpdateModel: aModel
	"The model is about to be updated.
	This is your chance to react about it."
	
	"no-op"
!!

onAboutToOpen

	"no-op"
!! !!

!!Presenter methodsFor: ''rendering''!!

renderOn: html
	"Render this presenter.
	Note: presenters are always wrapped in a div using as CSS class
	it''s class name."

	painted := true.

	wrapper := html div.
	
	wrapper
		class: self class name;
		with:[self paintOn: html]
!!

paintOn: html
	"Paints the content (the wrapped content, like wrapper NOT included)
	of this presenter. Sub classes should override as appropiate."
	
	"default is paint nothing"
	self
!!

paintOnJQuery: aJQuery
	"Appends what''s being paint by this presenter to aJQuery."

	self appendToJQuery: aJQuery
!! !!

!!Presenter methodsFor: ''testing''!!

hasParent

	^ parent notNil
!!

isPresenter

	^ true
!!

hasPresenterAt: aKey

	^ self children includesKey: aKey
!! !!

!!Presenter class methodsFor: ''accessing''!!

defaultModel

	^ nil
!! !!

!!Presenter class methodsFor: ''actions''!!

for: aModel

	^ self new
		model: aModel;
		yourself
!!

open

	^ (self for: self defaultModel) open
!! !!

Model subclass: #PersistentModel
	instanceVariableNames: ''id''
	category: ''Flow''!!
!!PersistentModel commentStamp!!
PersistentModels are objects that are manipulable here but they''re also stored server side.!!

!!PersistentModel methodsFor: ''accessing''!!

id: aString

	id := aString
!!

id

	^ id
!!

url

	^ self path,''/'',id asString
!!

path 

	^ self class path
!! !!

!!PersistentModel methodsFor: ''actions''!!

update

	jQuery ajax: #{ 
		''url'' -> self path, ''/'',id asString.
		''type'' -> ''PUT''.
		''data'' -> self asJSONString.
		''success'' -> [:x| self onAfterUpdate: x].
		''fail'' -> [:x| self onUpdateFail: x].
		''error'' -> [:x| self onUpdateFail: x]}
!!

delete

	jQuery ajax: #{ 
		''url'' -> (self path, ''/'',id asString).
		''type'' -> ''DELETE''.
		''success'' -> [:x| self onAfterDelete: x].
		''fail'' -> [:x| self onDeleteFail: x].
		''error'' -> [:x| self onDeleteFail: x]}
!!

syncWith: aReified
	"Sync the current values in this model 
	with the ones coming in aReifyed."
	| toSynchronize |

	toSynchronize := self class allInstanceVariableNames reject:[:e| self class jsonIgnoreInstanceVariables includes: e].
	toSynchronize do:[:instVarName|
		self 
			instVarAt: instVarName 
			put: (aReified at: instVarName)].
!!

refresh
	"Re-read this model''s state."

	jQuery ajax: #{ 
		''url'' -> (self path, ''/'',id asString).
		''type'' -> ''GET''.
		''success'' -> [:x| self onAfterRefresh: x].
		''fail'' -> [:x| self onRefeshFail: x].
		''error'' -> [:x| self onRefreshFail: x]}
!!

save
	^ self update
!! !!

!!PersistentModel methodsFor: ''reactions''!!

onAfterCreate: x

	self syncWith: (self class reify: x).
	self announce: (ModelCreated for: self)
!!

onAfterUpdate: x
	
	self announce: (ModelUpdated for: self)
!!

onUpdateFail: x

	ModelUpdateError signal: ''Could not update '', self class name
!!

onDeleteFail: x

	ModelUpdateError signal: ''Could not delete '', self class name
!!

onAfterDelete: x
	
	self announce: (ModelDeleted for: self)
!!

onRefreshFail: x

	ModelRefreshError signal: ''Could not refresh '', self class name
!!

onAfterRefresh: x

	self syncWith: (self class reify: x).
	self announce: (ModelRefreshed for: self)
!! !!

!!PersistentModel class methodsFor: ''accessing''!!

atId: anId
	"Answers the instance of this model found at anId (or nil)"
 
	^ self read: anId
!! !!

!!PersistentModel class methodsFor: ''actions''!!

read: anId

	jQuery ajax: #{ 
		''url'' -> (self path, ''/'',anId).
		''type'' -> ''GET''.
		''sucess'' -> [:x| self onAfterRead: x].
		''fail'' -> [:x| self onReadFail: x].
		''error'' -> [:x| self onReadFail: x]}
!!

create

	| newInstance |

	newInstance := self basicNew initialize.

	jQuery ajax: #{ 
		''url'' -> self path.
		''type'' -> ''POST''.
		''success'' -> [:x| newInstance onAfterCreate: x].
		''fail'' -> [:x| self onCreateFail: x].
		''error'' -> [:x| self onCreateFail: x]}.

	^ newInstance
!!

reify: aJSONString
	"Returns a simple javascript object with
	the attributes meant for the matching instance 
	of this model."

	^ JSON parse: aJSONString
!!

new
	
	^ self create
!! !!

!!PersistentModel class methodsFor: ''reactions''!!

onCreateFail: x

	ModelCreateError signal: ''Could not create '', self name,'':  '', x responseText
!!

onReadFail: x

	ModelReadError signal: ''Could not read '', self name,'':  '', x responseText
!!

onAfterRead: x

	^ self reify: x
!! !!

'! !
FlowAPIHandler initialize!
FlowCodeHandler initialize!
FlowAbstractLibrary initialize!
